//
//  Environment.cpp
//  runTileMap
//
//  Created by Mac on 14-8-18.
//
//

#include "Environment.h"
#include "Player.h"
#include "Partner.h"
#include "RoadBlock.h"
#include "Gold.h"
#include "Item.h"



CEnvironment::CEnvironment()
{
    m_fMoveDistance = 0;
    m_iStage        = 1;
    m_iMapIndex     = 1;
    
    pPartner = NULL;
    
    _iBridgeDuration = 0;
    _bShowBridge = false;
}

CEnvironment::~CEnvironment()
{
    for (int j=0; j<m_pEnvironmentSection_Array->count(); j++)
    {
        CEnvironmentVO* pSection = (CEnvironmentVO*)m_pEnvironmentSection_Array->objectAtIndex(j);
        for (int i=0; i<pSection->getGoldSetArray()->count(); i++){
            removeChild((CGold*)pSection->getGoldSetArray()->objectAtIndex(i),true);
        }
        for (int i=0; i<pSection->getRoadBlockArray()->count(); i++){
            removeChild((CRoadBlock*)pSection->getRoadBlockArray()->objectAtIndex(i),true);
        }
        for (int i=0; i<pSection->getItemArray()->count(); i++){
            removeChild((CItem*)pSection->getItemArray()->objectAtIndex(i),true);
        }
        removeChild(pSection->getTilemap(), true);
        m_pEnvironmentSection_Array->removeObjectAtIndex(0);
        CC_SAFE_DELETE(pSection);
    }    
    
    m_pEnvironmentSection_Array->removeAllObjects();
    
    removeChild(_pBatchNode_Gold1,true);
    removeChild(_pBatchNode_Gold5,true);
    removeChild(_pBatchNode_Gold10,true);
    removeChild(_pBatchNode_Gold20,true);
    removeChild(_pBatchNode_Gold40,true);
    
    removeChild(pPartner,true);
}

CEnvironment* CEnvironment::create(int iStage, int iMapIndex)
{
    CEnvironment* pEnvironment = new CEnvironment;
    if (pEnvironment && pEnvironment->init())
    {
        pEnvironment->initEnvironment(iStage, iMapIndex);
        pEnvironment->autorelease();
        return pEnvironment;
    }
    CC_SAFE_DELETE(pEnvironment);
    return NULL;
}

void CEnvironment::onEnter()
{
    CCLayer::onEnter();
    
    CCNotificationCenter::sharedNotificationCenter()->addObserver(this,
                                                                  callfuncO_selector(CEnvironment::onShowBridge),
                                                                  "showBridge",
                                                                  NULL);
}

void CEnvironment::onExit()
{
    CCLayer::onExit();
    CCNotificationCenter::sharedNotificationCenter()->removeObserver(this, "showBridge");
}

void CEnvironment::initEnvironment(int iStage, int iMapIndex)
{
    m_pEnvironmentSection_Array = CCArray::create();
    m_pEnvironmentSection_Array->retain();
    
//    CCSpriteFrameCache::sharedSpriteFrameCache()->spriteFrameByName(pszSpriteFrameName)
    
    CCTexture2D* Gold1Texture = CCSprite::createWithSpriteFrameName("gold1_1.png")->getTexture();
    _pBatchNode_Gold1 = CCSpriteBatchNode::createWithTexture(Gold1Texture, 100);
    addChild(_pBatchNode_Gold1, Terrain_Z+1);
    CCTexture2D* Gold5Texture =  CCSprite::createWithSpriteFrameName("gold5_1.png")->getTexture();
    _pBatchNode_Gold5 = CCSpriteBatchNode::createWithTexture(Gold5Texture, 100);
    addChild(_pBatchNode_Gold5, Terrain_Z+1);
    CCTexture2D* Gold10Texture =  CCSprite::createWithSpriteFrameName("gold10_1.png")->getTexture();
    _pBatchNode_Gold10 = CCSpriteBatchNode::createWithTexture(Gold10Texture, 100);
    addChild(_pBatchNode_Gold10, Terrain_Z+1);
    CCTexture2D* Gold20Texture =  CCSprite::createWithSpriteFrameName("gold20_1.png")->getTexture();
    _pBatchNode_Gold20 = CCSpriteBatchNode::createWithTexture(Gold20Texture, 100);
    addChild(_pBatchNode_Gold20, Terrain_Z+1);
    CCTexture2D* Gold40Texture =  CCSprite::createWithSpriteFrameName("gold40_1.png")->getTexture();
    _pBatchNode_Gold40 = CCSpriteBatchNode::createWithTexture(Gold40Texture, 100);
    addChild(_pBatchNode_Gold40, Terrain_Z+1);
    
    m_iStage = iStage;
    float fNextSectionPositionX = 0;
    for ( int Index = 0; Index< 2; Index++)
    {
        m_iMapIndex = iMapIndex + Index;
        CEnvironmentVO* pSection = new CEnvironmentVO;
        char szPath[128] = {0};
        sprintf(szPath, "Map/map%d_%d.tmx", m_iStage, m_iMapIndex);
        CCTMXTiledMap* _tileMap = CCTMXTiledMap::create(szPath);
        pSection->setTilemap(_tileMap);
        addChild(_tileMap, Terrain_Z);
         _tileMap->setPosition(CCPoint(fNextSectionPositionX, 0));
        //挡板层，默认不可见
        CCTMXLayer* pBoardLayer = _tileMap->layerNamed("board");
        if (pBoardLayer)
        {
            pBoardLayer->setVisible(false);
        }
        
        
        //地面碰撞
        CCTMXObjectGroup* objects = _tileMap->objectGroupNamed("roadBlock");
        CCArray* pTerrainCollisionArray = objects->getObjects();
        for (int i=0; i<pTerrainCollisionArray->count(); i++)
        {
            CCDictionary* collusionRect = (CCDictionary*)pTerrainCollisionArray->objectAtIndex(i);
            
                CCRect m_TerrainCollision_1 = CCRect(fNextSectionPositionX + collusionRect->valueForKey("x")->intValue(),
                                                     collusionRect->valueForKey("y")->intValue(),
                                                     collusionRect->valueForKey("width")->intValue(),
                                                     collusionRect->valueForKey("height")->intValue());
                
                pSection->m_TerrainCollisionVector.push_back(m_TerrainCollision_1);
            
        }
        //挡板
        objects = _tileMap->objectGroupNamed("roadBlock2");
        if (objects)
        {
            CCArray* pBoardCollisionArray = objects->getObjects();
            for (int i=0; i<pBoardCollisionArray->count(); i++)
            {
                CCDictionary* collusionRect = (CCDictionary*)pBoardCollisionArray->objectAtIndex(i);
                
                CCRect m_BoardCollision_1 = CCRect(fNextSectionPositionX + collusionRect->valueForKey("x")->intValue(),
                                                   collusionRect->valueForKey("y")->intValue(),
                                                   collusionRect->valueForKey("width")->intValue(),
                                                   collusionRect->valueForKey("height")->intValue());
                
                pSection->m_BoardCollisionVector.push_back(m_BoardCollision_1);
                
            }
        }
        
        
        //障碍
        objects = _tileMap->objectGroupNamed("stopBlock");
        if (objects)
        {
            char szTexture[128]={0};
            pTerrainCollisionArray = objects->getObjects();
            for (int i=0; i<pTerrainCollisionArray->count(); i++)
            {
                CCDictionary* pRoadBlockDictionary = (CCDictionary*)pTerrainCollisionArray->objectAtIndex(i);
                int iBlockType = pRoadBlockDictionary->valueForKey("name")->intValue();
                if (iBlockType == 4)
                {
                    int iBlockSubType = pRoadBlockDictionary->valueForKey("type")->intValue();
                    if (iBlockSubType ==1 ) {
                        sprintf(szTexture, "RoadBlock%d_1.png",iBlockType);
                    }
                    else if (iBlockSubType ==2){
                        sprintf(szTexture, "RoadBlock%d_1.png",iBlockType);
//                        setScale(2);
                        //不明白，在stage3 转过1时 变大大了
                    }
                    else if(iBlockSubType == 3){
                        sprintf(szTexture, "RoadBlock%d_3.png",iBlockType);
                    }
                }
                else
                {
                    sprintf(szTexture, "RoadBlock%d_1.png",iBlockType);
                }
                CRoadBlock* pRoadBlock = CRoadBlock::create(szTexture,iBlockType);
                if (iBlockType ==2 ) {
                    pRoadBlock->setPosition(ccp(fNextSectionPositionX + pRoadBlockDictionary->valueForKey("x")->intValue(),
                                                pRoadBlockDictionary->valueForKey("y")->intValue() + 20));
                }
                else{
                pRoadBlock->setPosition(ccp(fNextSectionPositionX + pRoadBlockDictionary->valueForKey("x")->intValue(),
                                            pRoadBlockDictionary->valueForKey("y")->intValue()));
                }
                addChild(pRoadBlock,Terrain_Z);
                pSection->getRoadBlockArray()->addObject(pRoadBlock);
            }
        }
        
        //金币
        objects = _tileMap->objectGroupNamed("gold2");
        if (objects) {
            char szTexture[128]={0};
            pTerrainCollisionArray = objects->getObjects();
            for (int i=0; i<pTerrainCollisionArray->count(); i++)
            {
                CCDictionary* pGoldDictionary = (CCDictionary*)pTerrainCollisionArray->objectAtIndex(i);
                
                int iType = pGoldDictionary->valueForKey("name")->intValue();
                sprintf(szTexture, "gold%d_1.png",iType);
                CGold* pGold = CGold::create(szTexture,iType);
                pGold->setPosition(ccp(fNextSectionPositionX + pGoldDictionary->valueForKey("x")->intValue(),
                                              pGoldDictionary->valueForKey("y")->intValue()));
                switch (iType)
                {
                    case 1:
                        _pBatchNode_Gold1->addChild(pGold);
                        break;
                    case 5:
                        _pBatchNode_Gold5->addChild(pGold);
                        break;
                    case 10:
                        _pBatchNode_Gold10->addChild(pGold);
                        break;
                    case 20:
                        _pBatchNode_Gold20->addChild(pGold);
                        break;
                    case 40:
                        _pBatchNode_Gold40->addChild(pGold);
                        break;
                    default:
                        break;
                }
                pSection->getGoldSetArray()->addObject(pGold);
            }
        }

        //道具
        objects = _tileMap->objectGroupNamed("item");
        if (objects) {
            char szTexture[128]={0};
            pTerrainCollisionArray = objects->getObjects();
            for (int i=0; i<pTerrainCollisionArray->count(); i++)
            {
                CCDictionary* pGoldDictionary = (CCDictionary*)pTerrainCollisionArray->objectAtIndex(i);
                
                int iType = pGoldDictionary->valueForKey("name")->intValue();
                
                if (iType == 6)
                {
                    if (pPartner == NULL)
                    {
                        std::vector<string> Action_Vector;
                        std::vector<float>	Delay_Vector;
                        
                        Action_Vector.push_back("Run_");
                        Delay_Vector.push_back(0.03);
                        Action_Vector.push_back("Jump_");
                        Delay_Vector.push_back(0.15);
                        Action_Vector.push_back("Fall_");
                        Delay_Vector.push_back(0.07);
                        Action_Vector.push_back("Roll_");
                        Delay_Vector.push_back(0.07);
                        Action_Vector.push_back("Slip_");
                        Delay_Vector.push_back(0.07);
                        Action_Vector.push_back("KnockDown_");
                        Delay_Vector.push_back(0.07);
                        
                        Action_Vector.push_back("TailRun_");
                        Delay_Vector.push_back(0.07);
                        Action_Vector.push_back("TailJump_");
                        Delay_Vector.push_back(0.07);
                        Action_Vector.push_back("TailFall_");
                        Delay_Vector.push_back(0.05);
                        Action_Vector.push_back("TailSlip_");
                        Delay_Vector.push_back(0.07);
                        
                        Action_Vector.push_back("EyesRun_");
                        Delay_Vector.push_back(0.07);
                        Action_Vector.push_back("EyesSlip_");
                        Delay_Vector.push_back(0.07);
                        
                        pPartner = CPartner::createPartner("Character2_", Action_Vector, Delay_Vector, 15);
                        addChild(pPartner,Player_Z);
                        pPartner->setAnchorPoint(ccp(0.5f,0.4f));
                    }
                    pPartner->setAttatch(false);
                    pPartner->setMoveDistance(0);
                    pPartner->setPosition(ccp(fNextSectionPositionX + pGoldDictionary->valueForKey("x")->intValue(),
                                              pGoldDictionary->valueForKey("y")->intValue()));
                    pPartner->playAction("Run_",false);
                    
                    continue;
                }

                
                sprintf(szTexture, "Item%d_1.png",iType);
                CItem* pItem = CItem::create(szTexture,iType);
                pItem->setPosition(ccp(fNextSectionPositionX + pGoldDictionary->valueForKey("x")->intValue(),
                                       pGoldDictionary->valueForKey("y")->intValue()));
                addChild(pItem,Item_Z);
                pSection->getItemArray()->addObject(pItem);
                
                if (iType == 4)
                {
//                    CCSprite* pPatrolStation = CCSprite::createWithSpriteFrameName("patrolStation.png");
                    
                    CCSprite* pPatrolStation = CCSprite::create("patrolStation.png");
                    pPatrolStation->setAnchorPoint(ccp(0, 0));
                    pPatrolStation->setPosition(ccp(fNextSectionPositionX - 50
                                                    + pGoldDictionary->valueForKey("x")->intValue(),
                                                    pGoldDictionary->valueForKey("y")->intValue()));
                    pPatrolStation->setScale(2);
                    addChild(pPatrolStation, Item_Z-2);
                }
                
            }
        }
        
        m_pEnvironmentSection_Array->addObject(pSection);
        
        fNextSectionPositionX += _tileMap->getMapSize().width *_tileMap->getTileSize().width;
        
    }
}

void CEnvironment::move (CPlayer* player)
{
    float xMove = player->getVector().x;
    this->setPosition(getPositionX() - xMove, 0);
    
    m_fMoveDistance += xMove;
    
    CEnvironmentVO* pSection = (CEnvironmentVO*)m_pEnvironmentSection_Array->objectAtIndex(1);
    
    if (m_fMoveDistance > pSection->getTilemap()->getPositionX())
    {
        m_iMapIndex++;
        //在21 ， 31 ， 41 ， 51 ， 61 ， 71转到宇宙场景
//        if (m_iStage != 3
//            &&(
//               m_iMapIndex == 12||//test
//             m_iMapIndex == 22
//            m_iMapIndex == 32
//            || m_iMapIndex == 42
//            || m_iMapIndex == 52
//            || m_iMapIndex == 62
//            || m_iMapIndex == 72
//               )
//            )
        if(m_iStage != 3
           && m_iMapIndex>10 &&
           m_iMapIndex%32==0)
        {
            CCNotificationCenter::sharedNotificationCenter()->postNotification("bonusMap");
            return;
        }
        
        if (m_iStage == 3 && m_iMapIndex > 3) {
            m_iMapIndex = 1;
            CCNotificationCenter::sharedNotificationCenter()->postNotification("completeBonusMap", CCInteger::create(1));
            CCNotificationCenter::sharedNotificationCenter()->postNotification("showBridge");
        }
        if (m_iMapIndex > 80) {
            m_iMapIndex = 1;
        }
        
        CCNotificationCenter::sharedNotificationCenter()->postNotification("updateMapIndex",CCInteger::create(m_iMapIndex));
        
        float fNextSectionPositionX = pSection->getTilemap()->getPositionX() +
                                      pSection->getTilemap()->getMapSize().width * pSection->getTilemap()->getTileSize().width;
        
        pSection = (CEnvironmentVO*)m_pEnvironmentSection_Array->objectAtIndex(0);
        for (int i=0; i<pSection->getGoldSetArray()->count(); i++){
            
            CGold* pGold = (CGold*)pSection->getGoldSetArray()->objectAtIndex(i);
            switch (pGold->getGoldType())
            {
                case 1:
                    _pBatchNode_Gold1->cocos2d::CCNode::removeChild((CGold*)pSection->getGoldSetArray()->objectAtIndex(i), true);
                    break;
                case 5:
                    _pBatchNode_Gold5->cocos2d::CCNode::removeChild((CGold*)pSection->getGoldSetArray()->objectAtIndex(i), true);
                    break;
                case 10:
                    _pBatchNode_Gold10->cocos2d::CCNode::removeChild((CGold*)pSection->getGoldSetArray()->objectAtIndex(i), true);
                    break;
                case 20:
                    _pBatchNode_Gold20->cocos2d::CCNode::removeChild((CGold*)pSection->getGoldSetArray()->objectAtIndex(i), true);
                    break;
                case 40:
                    _pBatchNode_Gold40->cocos2d::CCNode::removeChild((CGold*)pSection->getGoldSetArray()->objectAtIndex(i), true);
                    break;
                default:
                    break;
            }
        }
        for (int i=0; i<pSection->getRoadBlockArray()->count(); i++){
            removeChild((CRoadBlock*)pSection->getRoadBlockArray()->objectAtIndex(i),true);
        }
        for (int i=0; i<pSection->getItemArray()->count(); i++){
            removeChild((CItem*)pSection->getItemArray()->objectAtIndex(i),true);
        }
        pSection->getTilemap()->removeAllChildrenWithCleanup(true);
        removeChild(pSection->getTilemap(), true);
        m_pEnvironmentSection_Array->removeObjectAtIndex(0);
        CC_SAFE_DELETE(pSection);
        
        //地面碰撞
        CEnvironmentVO* pNewSection = new CEnvironmentVO;
        char szPath[128] = {0};
        sprintf(szPath, "Map/map%d_%d.tmx", m_iStage, m_iMapIndex);
        CCTMXTiledMap* _tileMap = CCTMXTiledMap::create(szPath);
        pNewSection->setTilemap(_tileMap);
        addChild(_tileMap, Terrain_Z);
        _tileMap->setPosition(CCPoint(fNextSectionPositionX, 0));
        //挡板层，默认不可见
        CCTMXLayer* pBoardLayer = _tileMap->layerNamed("board");
        if (pBoardLayer)
        {
            pBoardLayer->setVisible(_bShowBridge);
        }
        
        CCTMXObjectGroup* objects = _tileMap->objectGroupNamed("roadBlock");
        CCArray* pTerrainCollisionArray = objects->getObjects();
        for (int i=0; i<pTerrainCollisionArray->count(); i++)
        {
            CCDictionary* collusionRect = (CCDictionary*)pTerrainCollisionArray->objectAtIndex(i);
            
            CCRect m_TerrainCollision_1 = CCRect(fNextSectionPositionX + collusionRect->valueForKey("x")->intValue(),
                                                 collusionRect->valueForKey("y")->intValue(),
                                                 collusionRect->valueForKey("width")->intValue(),
                                                 collusionRect->valueForKey("height")->intValue());
            
            pNewSection->m_TerrainCollisionVector.push_back(m_TerrainCollision_1);            
        }
        
        
        //障碍
        objects = _tileMap->objectGroupNamed("stopBlock");
        if (objects)
        {
            char szTexture[128]={0};
            pTerrainCollisionArray = objects->getObjects();
            for (int i=0; i<pTerrainCollisionArray->count(); i++)
            {
                CCDictionary* pRoadBlockDictionary = (CCDictionary*)pTerrainCollisionArray->objectAtIndex(i);
                int iBlockType = pRoadBlockDictionary->valueForKey("name")->intValue();
                int iBlockSubType = 1;
                if (iBlockType == 4)
                {
                    iBlockSubType = pRoadBlockDictionary->valueForKey("type")->intValue();
                    if (iBlockSubType ==1 ) {
                        sprintf(szTexture, "RoadBlock%d_1.png",iBlockType);
                    }
                    else if (iBlockSubType ==2){
                        sprintf(szTexture, "RoadBlock%d_1.png",iBlockType);
                    }
                    else if(iBlockSubType == 3){
                        sprintf(szTexture, "RoadBlock%d_3.png",iBlockType);
                    }
                }
                else
                {
                    sprintf(szTexture, "RoadBlock%d_1.png",iBlockType);
                }
                CRoadBlock* pRoadBlock = CRoadBlock::create(szTexture,iBlockType,iBlockSubType);
                if (iBlockType ==2 ) {
                    pRoadBlock->setPosition(ccp(fNextSectionPositionX + pRoadBlockDictionary->valueForKey("x")->intValue(),
                                                pRoadBlockDictionary->valueForKey("y")->intValue() + 20));
                }
                else if (iBlockType ==5 ) {
                    pRoadBlock->setPosition(ccp(fNextSectionPositionX + pRoadBlockDictionary->valueForKey("x")->intValue(),
                                                pRoadBlockDictionary->valueForKey("y")->intValue() + 20));
                }
                else{
                    pRoadBlock->setPosition(ccp(fNextSectionPositionX + pRoadBlockDictionary->valueForKey("x")->intValue(),
                                                pRoadBlockDictionary->valueForKey("y")->intValue()));
                }
                if (iBlockType == 4){
                    addChild(pRoadBlock,Terrain_Z+10);
                }
                else{
                    addChild(pRoadBlock,Terrain_Z);
                }
                pNewSection->getRoadBlockArray()->addObject(pRoadBlock);
            }
        }
        
        //挡板
        objects = _tileMap->objectGroupNamed("roadBlock2");
        if (objects)
        {
            CCArray* pBoardCollisionArray = objects->getObjects();
            for (int i=0; i<pBoardCollisionArray->count(); i++)
            {
                CCDictionary* collusionRect = (CCDictionary*)pBoardCollisionArray->objectAtIndex(i);
                
                CCRect m_BoardCollision_1 = CCRect(fNextSectionPositionX + collusionRect->valueForKey("x")->intValue(),
                                                   collusionRect->valueForKey("y")->intValue(),
                                                   collusionRect->valueForKey("width")->intValue(),
                                                   collusionRect->valueForKey("height")->intValue());
                
                pNewSection->m_BoardCollisionVector.push_back(m_BoardCollision_1);
                
            }
        }

        //金币
        objects = _tileMap->objectGroupNamed("gold2");
        if (objects) {
            char szTexture[128]={0};
            pTerrainCollisionArray = objects->getObjects();
            for (int i=0; i<pTerrainCollisionArray->count(); i++)
            {
                CCDictionary* pGoldDictionary = (CCDictionary*)pTerrainCollisionArray->objectAtIndex(i);
                
                int iType = pGoldDictionary->valueForKey("name")->intValue();
                sprintf(szTexture, "gold%d_1.png",iType);
                CGold* pGold = CGold::create(szTexture,iType);
                pGold->setPosition(ccp(fNextSectionPositionX + pGoldDictionary->valueForKey("x")->intValue(),
                                       pGoldDictionary->valueForKey("y")->intValue()));
                switch (iType)
                {
                    case 1:
                        _pBatchNode_Gold1->addChild(pGold);
                        break;
                    case 5:
                        _pBatchNode_Gold5->addChild(pGold);
                        break;
                    case 10:
                        _pBatchNode_Gold10->addChild(pGold);
                        break;
                    case 20:
                        _pBatchNode_Gold20->addChild(pGold);
                        break;
                    case 40:
                        _pBatchNode_Gold40->addChild(pGold);
                        break;
                    default:
                        break;
                }
                pNewSection->getGoldSetArray()->addObject(pGold);
            }
        }
        
        //道具
        objects = _tileMap->objectGroupNamed("item");
        if (objects)
        {
            char szTexture[128]={0};
            pTerrainCollisionArray = objects->getObjects();
            for (int i=0; i<pTerrainCollisionArray->count(); i++)
            {
                CCDictionary* pGoldDictionary = (CCDictionary*)pTerrainCollisionArray->objectAtIndex(i);
                int iType = pGoldDictionary->valueForKey("name")->intValue();
                
//                else if(m_iMapIndex > 20)
//                {
                if (iType != 4 && iType !=5)
                {
                    iType = CCRANDOM_0_1() * 100;
                    iType = iType%4 +1;
                    
                    if (iType == 4) {
                        iType = 6;
                    }
                }
//                }
                
                if (iType == 6)
                {
                    if (pPartner == NULL)
                    {
                        std::vector<string> Action_Vector;
                        std::vector<float>	Delay_Vector;
                        
                        Action_Vector.push_back("Run_");
                        Delay_Vector.push_back(0.03);
                        Action_Vector.push_back("Jump_");
                        Delay_Vector.push_back(0.15);
                        Action_Vector.push_back("Fall_");
                        Delay_Vector.push_back(0.07);
                        Action_Vector.push_back("Roll_");
                        Delay_Vector.push_back(0.07);
                        Action_Vector.push_back("Slip_");
                        Delay_Vector.push_back(0.07);
                        Action_Vector.push_back("KnockDown_");
                        Delay_Vector.push_back(0.07);
                        
                        Action_Vector.push_back("TailRun_");
                        Delay_Vector.push_back(0.07);
                        Action_Vector.push_back("TailJump_");
                        Delay_Vector.push_back(0.07);
                        Action_Vector.push_back("TailFall_");
                        Delay_Vector.push_back(0.05);
                        Action_Vector.push_back("TailSlip_");
                        Delay_Vector.push_back(0.07);
                        
                        Action_Vector.push_back("EyesRun_");
                        Delay_Vector.push_back(0.07);
                        Action_Vector.push_back("EyesSlip_");
                        Delay_Vector.push_back(0.07);
                        
                        pPartner = CPartner::createPartner("Character2_", Action_Vector, Delay_Vector, 15);
                        addChild(pPartner,Player_Z);
                        pPartner->setAnchorPoint(ccp(0.5f,0.4f));
                    }
                    pPartner->setAttatch(false);
                    pPartner->setMoveDistance(0);
                    pPartner->setPosition(ccp(fNextSectionPositionX + pGoldDictionary->valueForKey("x")->intValue(),
                                              pGoldDictionary->valueForKey("y")->intValue()));
                    pPartner->playAction("Run_",false);
                    
                    continue;
                }
               
                sprintf(szTexture, "Item%d_1.png",iType);
                CItem* pItem = CItem::create(szTexture,iType);
                pItem->setPosition(ccp(fNextSectionPositionX + pGoldDictionary->valueForKey("x")->intValue(),
                                       pGoldDictionary->valueForKey("y")->intValue()));
                addChild(pItem,Item_Z);
                
                pNewSection->getItemArray()->addObject(pItem);               
                
            }
        }
        m_pEnvironmentSection_Array->addObject(pNewSection);
    }
    
    
    if (_bShowBridge == true)
    {
        _iBridgeDuration = 1800;
        
        for (int i=0; i<m_pEnvironmentSection_Array->count(); i++)
        {
            CEnvironmentVO* pSection = (CEnvironmentVO*)m_pEnvironmentSection_Array->objectAtIndex(i);
            CCTMXLayer* pBoardLayer = pSection->getTilemap()->layerNamed("board");
            if (pBoardLayer)
            {
                pBoardLayer->setVisible(true);
            }
            
        }
        _bShowBridge = false;
    }
//    else if (_iBridgeDuration>0)
//    {
//        _iBridgeDuration -= xMove;
//        if (_iBridgeDuration <=0)
//        {
//            _iBridgeDuration = 0;
//            _bShowBridge = false;
//            
//            for (int i=0; i<m_pEnvironmentSection_Array->count(); i++)
//            {
//                CEnvironmentVO* pSection = (CEnvironmentVO*)m_pEnvironmentSection_Array->objectAtIndex(i);
//                CCTMXLayer* pBoardLayer = pSection->getTilemap()->layerNamed("board");
//                if (pBoardLayer)
//                {
//                    pBoardLayer->setVisible(true);
//                }
//            }
//        }
//    }
    checkCollision(player);
}



void CEnvironment::checkCollision (CPlayer * player)
{
    CEnvironmentVO* pSection;
    
    if (m_pEnvironmentSection_Array->count() >= 2)
    {
        if (!player->getIsAccelerate())
        {
            //当主角和房子在Ｘ轴上时
            bool inAir = true;
            for (int iSection=0; iSection < 2; iSection++)
            {
                pSection = (CEnvironmentVO*)m_pEnvironmentSection_Array->objectAtIndex(iSection);
                
                for (int i=0; i< pSection->m_TerrainCollisionVector.size(); i++)
                {
                    if (player->right() >=  pSection->m_TerrainCollisionVector[i].getMinX() &&
                        player->left() <=  pSection->m_TerrainCollisionVector[i].getMaxX())
                    {
                        if (player->bottom() >= pSection->m_TerrainCollisionVector[i].getMaxY() &&
                            player->next_bottom() <= pSection->m_TerrainCollisionVector[i].getMaxY() &&
                            player->top() > pSection->m_TerrainCollisionVector[i].getMaxY())
                        {
                            player->setNextPosition(ccp(player->getNextPosition().x, pSection->m_TerrainCollisionVector[i].getMaxY()));
                            player->setVector ( ccp(player->getVector().x, 0) );
                            player->setGroundY(pSection->m_TerrainCollisionVector[i].getMaxY());
                            inAir = false;
                            break;
                        }
                    }
                }
                if (inAir)
                {
                    CEnvironmentVO* pSection = (CEnvironmentVO*)m_pEnvironmentSection_Array->objectAtIndex(iSection);
                    CCTMXLayer* pBoardLayer = pSection->getTilemap()->layerNamed("board");
                    if (pBoardLayer && pBoardLayer->isVisible())
                    {
                        for (int i=0; i<pSection->m_BoardCollisionVector.size(); i++)
                        {
                            if (player->right() >=  pSection->m_BoardCollisionVector[i].getMinX() &&
                                player->left() <=  pSection->m_BoardCollisionVector[i].getMaxX())
                            {
                                if (player->bottom() >= pSection->m_BoardCollisionVector[i].getMaxY() &&
                                    player->next_bottom() <= pSection->m_BoardCollisionVector[i].getMaxY() &&
                                    player->top() > pSection->m_BoardCollisionVector[i].getMaxY())
                                {
                                    player->setNextPosition(ccp(player->getNextPosition().x, pSection->m_BoardCollisionVector[i].getMaxY()));
                                    player->setVector ( ccp(player->getVector().x, 0) );
                                    player->setGroundY(pSection->m_TerrainCollisionVector[i].getMaxY());
                                    inAir = false;
                                    break;
                                }
                            }
                        }
                    }
                }
                
                if (inAir == false)
                {
                    break;
                }
            }
            
            //撞墙
            bool bHitWall = false;
            for (int iSection=0; iSection < 2; iSection++)
            {
                pSection = (CEnvironmentVO*)m_pEnvironmentSection_Array->objectAtIndex(iSection);
                for (int i=0; i<pSection->m_TerrainCollisionVector.size(); i++)
                {
                    if ((player->bottom() < pSection->m_TerrainCollisionVector[i].getMaxY() &&
                         player->top() > pSection->m_TerrainCollisionVector[i].getMinY()) ||
                        (player->next_bottom() < pSection->m_TerrainCollisionVector[i].getMaxY() &&
                         player->next_top() > pSection->m_TerrainCollisionVector[i].getMinY()))
                    {
                        if (player->right() >=  pSection->m_TerrainCollisionVector[i].getMinX()
                            && player->left() <  pSection->m_TerrainCollisionVector[i].getMinX())
                        {
                            player->setPositionX(  pSection->m_TerrainCollisionVector[i].getMinX() - player->getWidth() * 0.5f );
                            player->setNextPosition(ccp( pSection->m_TerrainCollisionVector[i].getMinX() - player->getWidth() * 0.5f, player->getNextPosition().y));
                            player->setVector ( ccp(player->getVector().x * -0.5f, player->getVector().y) );
                            //主角装墙上了
                            if (player->bottom() + player->getHeight() * 0.2f < pSection->m_TerrainCollisionVector[i].getMaxY() && player->getVector().y <=0)
                            {
                                player->setNextPosition(ccp(player->getNextPosition().x, player->getNextPosition().y));
                                player->setMoveSpeed(player->getMaxSpeed()*0.5f);
                                bHitWall = true;
                                break;
                            }
                        }
                    }
                }
                if (bHitWall)
                {
                    break;
                }
            }
            
            if (!bHitWall)
            {
                float fMoveDistance = player->getVector().x * 0.01f;
                CCNotificationCenter::sharedNotificationCenter()->postNotification("updateMoveDistance", CCFloat::create(fMoveDistance));
            }
            
            //撞陷阱
            for (int iSection=0; iSection < 2; iSection++)
            {
                pSection = (CEnvironmentVO*)m_pEnvironmentSection_Array->objectAtIndex(iSection);
                for (int j=0; j<pSection->getRoadBlockArray()->count(); j++)
                {
                    ((CRoadBlock*)pSection->getRoadBlockArray()->objectAtIndex(j))->update(player);
                    
                }
            }
            
            if (inAir)
            {
                //设置下降状态
                if (player->getPlayerState() != kPlayerFly)
                {
                    player->setPlayerState(kPlayerFalling);
                }
            }
            else
            {
                //设置一直移植状态
                if (player->getPlayerState() != kPlayerMoving) {
                    player->stopActionByTag(1);
                    player->playAction("Run_");
                    player->setPlayerAction(kActionMoving);
                }
                player->setPlayerState(kPlayerMoving);
            }
        }
        else
        {
            player->setNextPosition(ccp(player->getNextPosition().x, 450));
            player->setVector ( ccp(player->getVector().x, 0) );
        }
        
        
        //捡金币
        for (int i=0; i<2; i++)
        {
            pSection = (CEnvironmentVO*)m_pEnvironmentSection_Array->objectAtIndex(i);
            for (int j=0; j<pSection->getGoldSetArray()->count(); j++)
            {
                CGold* pGold = ((CGold*)pSection->getGoldSetArray()->objectAtIndex(j));
                pGold->update(player);
                
                if (player->getIsAbsorb())
                {
                    if(player->getCollectRect().intersectsRect(pGold->boundingBox()))
                    {
                        pGold->setAbosrb(true);
                    }
                    if (player->getCollisionRect().intersectsRect(((CCSprite*)pSection->getGoldSetArray()->objectAtIndex(j))->boundingBox()))
                    {
                        int iGold = ((CGold*)pSection->getGoldSetArray()->objectAtIndex(j))->getGoldType();
                        int iBonusGold =  player->getAbsorbLevel();
                        iGold = iGold + iBonusGold;
                        
                        switch (iGold)
                        {
                            case 1:
                                _pBatchNode_Gold1->removeChild((CGold*)pSection->getGoldSetArray()->objectAtIndex(j), true);
                                break;
                            case 5:
                                _pBatchNode_Gold5->removeChild((CGold*)pSection->getGoldSetArray()->objectAtIndex(j), true);
                                break;
                            case 10:
                                _pBatchNode_Gold10->removeChild((CGold*)pSection->getGoldSetArray()->objectAtIndex(j), true);
                                break;
                            case 20:
                                _pBatchNode_Gold20->removeChild((CGold*)pSection->getGoldSetArray()->objectAtIndex(j), true);
                                break;
                            case 40:
                                _pBatchNode_Gold40->removeChild((CGold*)pSection->getGoldSetArray()->objectAtIndex(j), true);
                                break;
                            default:
                                break;
                        }
                        pSection->getGoldSetArray()->removeObjectAtIndex(j);
                        
                        player->getGold();
                                                
                        if (pPartner && pPartner->getAttatch()) {
                            iGold = iGold*2;
                        }
                        CCNotificationCenter::sharedNotificationCenter()->postNotification("updateGoldeBar", CCInteger::create(iGold));
                    }
                }
                else
                {
                    if(player->getCollectRect().intersectsRect(((CCSprite*)pSection->getGoldSetArray()->objectAtIndex(j))->boundingBox()))
                    {
                        int iGold = ((CGold*)pSection->getGoldSetArray()->objectAtIndex(j))->getGoldType();
                        
                        switch (iGold)
                        {
                            case 1:
                                _pBatchNode_Gold1->removeChild((CGold*)pSection->getGoldSetArray()->objectAtIndex(j), true);
                                break;
                            case 5:
                                _pBatchNode_Gold5->removeChild((CGold*)pSection->getGoldSetArray()->objectAtIndex(j), true);
                                break;
                            case 10:
                                _pBatchNode_Gold10->removeChild((CGold*)pSection->getGoldSetArray()->objectAtIndex(j), true);
                                break;
                            case 20:
                                _pBatchNode_Gold20->removeChild((CGold*)pSection->getGoldSetArray()->objectAtIndex(j), true);
                                break;
                            case 40:
                                _pBatchNode_Gold40->removeChild((CGold*)pSection->getGoldSetArray()->objectAtIndex(j), true);
                                break;
                            default:
                                break;
                        }
                        pSection->getGoldSetArray()->removeObjectAtIndex(j);
                        player->getGold();
                        
                        if (pPartner && pPartner->getAttatch()) {
                            iGold = iGold*2;
                        }
                        CCNotificationCenter::sharedNotificationCenter()->postNotification("updateGoldeBar", CCInteger::create(iGold));
                    }
                }
            }
        }
        
        //捡道具
        for (int i=0; i<2; i++)
        {
            
            pSection = (CEnvironmentVO*)m_pEnvironmentSection_Array->objectAtIndex(i);
            for (int j=0; j<pSection->getItemArray()->count(); j++)
            {
                if(player->boundingBox().intersectsRect(((CItem*)pSection->getItemArray()->objectAtIndex(j))->getCollectRect()))
                {
                    int iItemType = ((CItem*)pSection->getItemArray()->objectAtIndex(j))->getItemType();
                    if (iItemType == 4) {
                        CItem* pItem = (CItem*)pSection->getItemArray()->objectAtIndex(j);
                        if (pItem) {
                            pItem->getItem();
                        }
                    }
                    else if (iItemType != 5) {
                        CItem* pItem = (CItem*)pSection->getItemArray()->objectAtIndex(j);
                        if (pItem) {
                            removeChild(pItem, true);
                        }
                    }
                    pSection->getItemArray()->removeObjectAtIndex(j);
                    
                    player->getItem(iItemType);
                }
                
            }
        }
        
        if (pPartner && player->boundingBox().intersectsRect(pPartner->boundingBox()))
        {
            pPartner->setAttatch(true);
        }
        if (pPartner && pPartner->getAttatch()) {
            pPartner->update(player);
        }
    }
    player->setMoveDistance(m_fMoveDistance);
}

void CEnvironment::onShowBridge()
{
    _bShowBridge = true;
}

void CEnvironment::hideBridge()
{
    _bShowBridge = false;
}

CPartner* CEnvironment::getPartner()
{
    return pPartner;
}