//
//  GameScene.cpp
//  runTileMap
//
//  Created by Mac on 14-8-14.
//
//

#include "GameScene.h"

#include "Player.h"
#include "Pet.h"
#include "Environment.h"
#include "ScoreScene.h"
#include "PauseLayer.h"
#include "SoundManager.h"

#include "GameModel.h"

//#define BackGround_Z 100
//#define Terrain_Z 200
//#define Player_Z 300
//#define Item_Z 250
//#define UI_Z 500

CGameScene::CGameScene()
{
    _screenSize = CCDirector::sharedDirector()->getWinSize();
    m_iGold = 0;
    m_iBonusGold = 0;
    m_fMoveDistance = 0.0f;
    m_iCountDown = 0;
    m_iResurrectionTime = 0;
    _iBGRepeatCount = 0;
    _iBGIndex = 1;
    _bChangeMap = false;
    _bBonusMap = false;
    _bCompleteBonusMap = false;
    m_bPopUpDistance = false;
    
    m_iRebornChance = 999;
    
    m_pPet = NULL;
    readyToSlip = false;
}

CGameScene::~CGameScene()
{
    
}

cocos2d::CCScene* CGameScene::scene()
{
    CCScene *scene = CCScene::create();
    CGameScene *layer = CGameScene::create();
    scene->addChild(layer);
    
    return scene;
}

bool CGameScene::init()
{
    if ( !CCLayer::init() )
    {
        return false;
    }
    
    isDoingNewHand1 = 0;
    isDoingNewHand2 = 0;
    
    CCSpriteFrameCache* cache = CCSpriteFrameCache::sharedSpriteFrameCache();
    
    cache->addSpriteFramesWithFile("GameUI.plist");    
//    cache->addSpriteFramesWithFile("RoadBlock.plist");
    cache->addSpriteFramesWithFile("Trap.plist");
    
    
    cache->addSpriteFramesWithFile("CharacterEffect.plist");
    cache->addSpriteFramesWithFile("Portal.plist");
    cache->addSpriteFramesWithFile("Gold.plist");
    cache->addSpriteFramesWithFile("backGround.plist");
    cache->addSpriteFramesWithFile("UUCharacter.plist");
    
    if (CGameData::getInstance()->getEffectEnable()) {
        cache->addSpriteFramesWithFile("Effect.plist");
        //cache->addSpriteFramesWithFile("ItemEffect.plist");
    }
    
    initEffect();
    
    initUI();
    
    initBackGround();
    
    //场景
    initEnvironment();
    
    //玩家
    initPlayer();
    
    //宠物
//    if (CGameData::getInstance()->getPetLevel())
//    {
//        initPet();
//        if (CGameData::getInstance()->getFood())
//        {
//            m_pPlayer->getItem(1);
//            CGameData::getInstance()->setFood(CGameData::getInstance()->getFood() - 1);
//        }
//    }
    if (CGameData::getInstance()->getPet1() > 0 || CGameData::getInstance()->getPet2() > 0 || CGameData::getInstance()->getPet3() > 0 ) {
        
            initPet();
    }
    else if (CGameData::getInstance()->getShell())
    {
        m_pPlayer->getShield(1);
        CGameData::getInstance()->setShell(CGameData::getInstance()->getShell() - 1);
    }
    
    if (CGameData::getInstance()->getEffectEnable())
    {
        CCParticleSystemQuad* m_pEmitter = CCParticleSystemQuad::create("speedLine.plist");         //粒粒嘢
        this->addChild(m_pEmitter,UI_Z);
        m_pEmitter->setPosition(ccp(_screenSize.width - 200, _screenSize.height*0.5f));
        
        CCParticleSystemQuad* m_pSpeedEmitter1 = CCParticleSystemQuad::create("speedLine12.plist");
        this->addChild(m_pSpeedEmitter1, UI_Z);
        m_pSpeedEmitter1->setPosition(ccp(_screenSize.width, _screenSize.height*0.5f + 300));
        
        CCParticleSystemQuad* m_pSpeedEmitter2 = CCParticleSystemQuad::create("speedLine12.plist"); //加速线
        this->addChild(m_pSpeedEmitter2, UI_Z);
        m_pSpeedEmitter2->setPosition(ccp(_screenSize.width, _screenSize.height*0.5f - 300));
    }
    
    this->scheduleUpdate();
    
    this->setTouchEnabled(true);
    
    CSoundManager::getInstance()->playMusic("sound/BRYYX_1.mp3");
    
    
    
    
    return true;
}

void CGameScene::initEnvironment()
{
    CCArray *animFrames = CCArray::create();
    animFrames->removeAllObjects();
    
    //路障1-刀锋
    char str[256] = {0};
    for(int j = 1; j < 10; j++)
    {
        sprintf(str, "RoadBlock1_%d.png", j);
        CCSpriteFrame* frame = CCSpriteFrameCache::sharedSpriteFrameCache()->spriteFrameByName(str);
        if(!frame) break;
        animFrames->addObject(frame);
    }
    CCAnimation* animation11 = CCAnimation::createWithSpriteFrames(animFrames,0.1);
    CCAnimationCache::sharedAnimationCache()->addAnimation(animation11, "RoadBlock1_");

    //路障2-倒刺
    animFrames->removeAllObjects();
    for(int j = 1; j < 9; j++)
    {
        sprintf(str, "RoadBlock2_%d.png", j);
        CCSpriteFrame* frame = CCSpriteFrameCache::sharedSpriteFrameCache()->spriteFrameByName(str);
        if(!frame) break;
        animFrames->addObject(frame);
    }
    CCAnimation* animation21 = CCAnimation::createWithSpriteFrames(animFrames,0.15);
    CCAnimationCache::sharedAnimationCache()->addAnimation(animation21, "RoadBlock2_");
    
    //路障3-冰刺
    animFrames->removeAllObjects();
    for(int j = 1; j < 9; j++)
    {
        sprintf(str, "RoadBlock3_%d.png", j);
        CCSpriteFrame* frame = CCSpriteFrameCache::sharedSpriteFrameCache()->spriteFrameByName(str);
        if(!frame) break;
        animFrames->addObject(frame);
    }
    CCAnimation* animation31 = CCAnimation::createWithSpriteFrames(animFrames,0.07);
    CCAnimationCache::sharedAnimationCache()->addAnimation(animation31, "RoadBlock3_");
        
    animFrames->removeAllObjects();
    for(int j = 1; j < 9; j++)
    {
        sprintf(str, "RoadBlockCollapse3_%d.png", j);
        CCSpriteFrame* frame = CCSpriteFrameCache::sharedSpriteFrameCache()->spriteFrameByName(str);
        if(!frame) break;
        animFrames->addObject(frame);
    }
    CCAnimation* animation33 = CCAnimation::createWithSpriteFrames(animFrames,0.07);
    CCAnimationCache::sharedAnimationCache()->addAnimation(animation33, "RoadBlockCollapse3_");
    
    //路障4-炮塔子弹
    animFrames->removeAllObjects();
    for(int j = 1; j < 15; j++)
    {
        sprintf(str, "Bullet_1_Effect%d.png", j);
        CCSpriteFrame* frame = CCSpriteFrameCache::sharedSpriteFrameCache()->spriteFrameByName(str);
        if(!frame) break;
        animFrames->addObject(frame);
    }
    CCAnimation* animation41 = CCAnimation::createWithSpriteFrames(animFrames,0.07);
    CCAnimationCache::sharedAnimationCache()->addAnimation(animation41, "Bullet1_");
    
    //路障5
    animFrames->removeAllObjects();
    for(int j = 1; j < 15; j++)
    {
        sprintf(str, "RoadBlock5_%d.png", j);
        CCSpriteFrame* frame = CCSpriteFrameCache::sharedSpriteFrameCache()->spriteFrameByName(str);
        if(!frame) break;
        animFrames->addObject(frame);
    }
    CCAnimation* animation51 = CCAnimation::createWithSpriteFrames(animFrames,0.07);
    CCAnimationCache::sharedAnimationCache()->addAnimation(animation51, "RoadBlock5_");
    
    animFrames->removeAllObjects();
    for(int j = 1; j < 9; j++)
    {
        sprintf(str, "RoadBlock5_Drill%d.png", j);
        CCSpriteFrame* frame = CCSpriteFrameCache::sharedSpriteFrameCache()->spriteFrameByName(str);
        if(!frame) break;
        animFrames->addObject(frame);
    }
    CCAnimation* animation52 = CCAnimation::createWithSpriteFrames(animFrames,0.07);
    CCAnimationCache::sharedAnimationCache()->addAnimation(animation52, "RoadBlock5_Drill");
    
    //金币1
    animFrames->removeAllObjects();
    for(int j = 1; j < 15; j++)
    {
        sprintf(str, "gold1_%d.png", j);
        CCSpriteFrame* frame = CCSpriteFrameCache::sharedSpriteFrameCache()->spriteFrameByName(str);
        if(!frame) break;
        animFrames->addObject(frame);
    }
    CCAnimation* animGold1 = CCAnimation::createWithSpriteFrames(animFrames,0.3);
    CCAnimationCache::sharedAnimationCache()->addAnimation(animGold1, "Gold_1");
    
    //金币5
    animFrames->removeAllObjects();
    for(int j = 1; j < 15; j++)
    {
        sprintf(str, "gold5_%d.png", j);
        CCSpriteFrame* frame = CCSpriteFrameCache::sharedSpriteFrameCache()->spriteFrameByName(str);
        if(!frame) break;
        animFrames->addObject(frame);
    }
    CCAnimation* animGold5 = CCAnimation::createWithSpriteFrames(animFrames,0.3);
    CCAnimationCache::sharedAnimationCache()->addAnimation(animGold5, "Gold_5");
    
    //金币10
    animFrames->removeAllObjects();
    for(int j = 1; j < 15; j++)
    {
        sprintf(str, "gold10_%d.png", j);
        CCSpriteFrame* frame = CCSpriteFrameCache::sharedSpriteFrameCache()->spriteFrameByName(str);
        if(!frame) break;
        animFrames->addObject(frame);
    }
    CCAnimation* animGold10 = CCAnimation::createWithSpriteFrames(animFrames,0.3);
    CCAnimationCache::sharedAnimationCache()->addAnimation(animGold10, "Gold_10");
    
    //金币20
    animFrames->removeAllObjects();
    for(int j = 1; j < 15; j++)
    {
        sprintf(str, "gold20Effect_%d.png", j);
        CCSpriteFrame* frame = CCSpriteFrameCache::sharedSpriteFrameCache()->spriteFrameByName(str);
        if(!frame) break;
        animFrames->addObject(frame);
    }
    CCAnimation* animGold20 = CCAnimation::createWithSpriteFrames(animFrames,0.5);
    CCAnimationCache::sharedAnimationCache()->addAnimation(animGold20, "Gold20_Effect");
    
    //金币40
    animFrames->removeAllObjects();
    for(int j = 1; j < 15; j++)
    {
        sprintf(str, "gold40Effect_%d.png", j);
        CCSpriteFrame* frame = CCSpriteFrameCache::sharedSpriteFrameCache()->spriteFrameByName(str);
        if(!frame) break;
        animFrames->addObject(frame);
    }
    CCAnimation* animGold40 = CCAnimation::createWithSpriteFrames(animFrames,0.5);
    CCAnimationCache::sharedAnimationCache()->addAnimation(animGold40, "Gold40_Effect");

    //传送门
    animFrames->removeAllObjects();
    for(int j = 1; j < 15; j++)
    {
        sprintf(str, "Portal_%d.png", j);
        CCSpriteFrame* frame = CCSpriteFrameCache::sharedSpriteFrameCache()->spriteFrameByName(str);
        if(!frame) break;
        animFrames->addObject(frame);
    }
    CCAnimation* portal = CCAnimation::createWithSpriteFrames(animFrames,0.2);
    CCAnimationCache::sharedAnimationCache()->addAnimation(portal, "Portal");
    
    //电池
    animFrames->removeAllObjects();
    for(int j = 1; j < 15; j++)
    {
        sprintf(str, "Item4_Effect%d.png", j);
        CCSpriteFrame* frame = CCSpriteFrameCache::sharedSpriteFrameCache()->spriteFrameByName(str);
        if(!frame) break;
        animFrames->addObject(frame);
    }
    CCAnimation* battery = CCAnimation::createWithSpriteFrames(animFrames,0.1);
    CCAnimationCache::sharedAnimationCache()->addAnimation(battery, "Battery");
    
    //导弹
    animFrames->removeAllObjects();
    for(int j = 1; j < 15; j++)
    {
        sprintf(str, "Item1_Effect%d.png", j);
        CCSpriteFrame* frame = CCSpriteFrameCache::sharedSpriteFrameCache()->spriteFrameByName(str);
        if(!frame) break;
        animFrames->addObject(frame);
    }
    CCAnimation* missile = CCAnimation::createWithSpriteFrames(animFrames,0.1);
    CCAnimationCache::sharedAnimationCache()->addAnimation(missile, "Missile");
    
    //磁铁
    animFrames->removeAllObjects();
    for(int j = 1; j < 15; j++)
    {
        sprintf(str, "Item2_Effect%d.png", j);
        CCSpriteFrame* frame = CCSpriteFrameCache::sharedSpriteFrameCache()->spriteFrameByName(str);
        if(!frame) break;
        animFrames->addObject(frame);
    }
    CCAnimation* magnet = CCAnimation::createWithSpriteFrames(animFrames,0.1);
    CCAnimationCache::sharedAnimationCache()->addAnimation(magnet, "Magnet");
    
    //盾牌
    animFrames->removeAllObjects();
    for(int j = 1; j < 15; j++)
    {
        sprintf(str, "Item3_Effect%d.png", j);
        CCSpriteFrame* frame = CCSpriteFrameCache::sharedSpriteFrameCache()->spriteFrameByName(str);
        if(!frame) break;
        animFrames->addObject(frame);
    }
    CCAnimation* shell = CCAnimation::createWithSpriteFrames(animFrames,0.1);
    CCAnimationCache::sharedAnimationCache()->addAnimation(shell, "Shell");
    
    m_pEnvironment = CEnvironment::create(1);
    addChild(m_pEnvironment, Terrain_Z);
    
}

void CGameScene::initPlayer()
{
    std::vector<string> Action_Vector;
	std::vector<float>	Delay_Vector;
    
    Action_Vector.push_back("Run_");
    Delay_Vector.push_back(0.03);
    Action_Vector.push_back("Jump_");
    Delay_Vector.push_back(0.15);
    Action_Vector.push_back("Fall_");
    Delay_Vector.push_back(0.07);
    Action_Vector.push_back("Roll_");
    Delay_Vector.push_back(0.07);
    Action_Vector.push_back("Slip_");
    Delay_Vector.push_back(0.07);
    Action_Vector.push_back("KnockDown_");
    Delay_Vector.push_back(0.07);
    Action_Vector.push_back("Shock_");
    Delay_Vector.push_back(0.07);
    Action_Vector.push_back("Die_");
    Delay_Vector.push_back(0.07);
    Action_Vector.push_back("Burn_");
    Delay_Vector.push_back(0.07);
    
    Action_Vector.push_back("TailRun_");
    Delay_Vector.push_back(0.07);
    Action_Vector.push_back("TailJump_");
    Delay_Vector.push_back(0.07);
    Action_Vector.push_back("TailFall_");
    Delay_Vector.push_back(0.05);
    Action_Vector.push_back("TailSlip_");
    Delay_Vector.push_back(0.07);
    
    Action_Vector.push_back("EyesRun_");
    Delay_Vector.push_back(0.07);
    Action_Vector.push_back("EyesSlip_");
    Delay_Vector.push_back(0.07);
    
    Action_Vector.push_back("IronRun_");
    Delay_Vector.push_back(0.03);
    Action_Vector.push_back("IronJump_");
    Delay_Vector.push_back(0.15);
    Action_Vector.push_back("IronFall_");
    Delay_Vector.push_back(0.07);
    Action_Vector.push_back("IronRoll_");
    Delay_Vector.push_back(0.07);
    Action_Vector.push_back("IronSlip_");
    Delay_Vector.push_back(0.07);
    
    std::vector<string> Effect_Vector;
    std::vector<float>	EffectDelay_Vector;
   
    Effect_Vector.push_back("EffectDust_");
    EffectDelay_Vector.push_back(0.07);
    Effect_Vector.push_back("EffectJump_");
    EffectDelay_Vector.push_back(0.05);
    Effect_Vector.push_back("EffectDoubleJump_");
    EffectDelay_Vector.push_back(0.07);
    
    m_pPlayer = CPlayer::createPlayer("Character1_");
    m_pPlayer->setData("Character1", 1, 3, 0.025, 10);
    m_pPlayer->initAnimation(Action_Vector, Delay_Vector, 15);
    m_pPlayer->initEfect(Effect_Vector, EffectDelay_Vector, 15);
    m_pEnvironment->addChild(m_pPlayer,Player_Z);
    m_pPlayer->setAnchorPoint(ccp(0.5f,0.4f));
    m_pPlayer->playAction("Fall_",false);

    CCPoint vBornPosition = ccp(20,640);
    m_pPlayer->setPosition(vBornPosition);
    
    if (CGameData::getInstance()->getEffectEnable())
    {
        this->schedule(schedule_selector(CGameScene::updateRunEffect), 1.5);
    }
    
}

void CGameScene::initPet()
{
    
    PETTYPE2  pettype = PET1;
    
    if (CGameData::getInstance()->getPet3() > 0) {
        GameModel::getInstance()->usingPet = 3;
        CGameData::getInstance() ->setPet3( CGameData::getInstance()->getPet3() - 1);
        pettype = PET3;
    }else if (CGameData::getInstance()->getPet2() > 0) {
        GameModel::getInstance()->usingPet = 2;
        CGameData::getInstance() ->setPet3( CGameData::getInstance()->getPet2() - 1);
        pettype = PET2;
    }else if (CGameData::getInstance()->getPet1() > 0) {
        GameModel::getInstance()->usingPet = 1;
        CGameData::getInstance() ->setPet3( CGameData::getInstance()->getPet1() - 1);
        pettype = PET1;
    }
    
    m_pPlayer->getItem(1);
    
    std::vector<string> Action_Vector;
	std::vector<float>	Delay_Vector;
    
    char cPethead[64];
    sprintf(cPethead, "Pet%d_",3);
    
    
    Action_Vector.push_back("Run_");
    
    Delay_Vector.push_back(0.04);
    
    Action_Vector.push_back("TailRun_");
    Delay_Vector.push_back(0.03);
    
    Action_Vector.push_back("EyesRun_");
    Delay_Vector.push_back(0.07);
    
    m_pPet = CPet::createPet(cPethead,Action_Vector, Delay_Vector, 15, AirType);
    
    m_pPet->playAction("Run_");
    m_pPet->setPosition(ccp(600,300));
    addChild(m_pPet, Player_Z);
}

void CGameScene::initBackGround()
{
    CCSprite * repeat;
    
    _clound = CCSpriteBatchNode::create("back1_4.jpg");
    _clound->setAnchorPoint(ccp(0,0));
    _clound->setScale(2);
    _clound->setPosition(CCPointZero);
    addChild(_clound, BackGround_Z);
    
    repeat = CCSprite::create("back1_4.jpg");
    repeat->setAnchorPoint(ccp(0,0));
    repeat->setPosition(ccp(0, 0));
    _clound->addChild(repeat);
    
    repeat = CCSprite::create("back1_4.jpg");
    repeat->setAnchorPoint(ccp(0,0));
    repeat->setPosition(ccp(repeat->getContentSize().width - 1, 0));
    _clound->addChild(repeat);
    
    CCSize _cloundContentSize = repeat->getContentSize();
    _clound->setContentSize(CCSizeMake(_cloundContentSize.width*2, _cloundContentSize.height*2));
    
    CCSprite* backgroundSprite = CCSprite::createWithSpriteFrameName("back1_5.png");
    _background = CCSpriteBatchNode::createWithTexture(backgroundSprite->getTexture(),3);
    _background->setAnchorPoint(ccp(0,0));
    _background->setScale(2);
    _background->setPosition(CCPointZero);
    addChild(_background, BackGround_Z);
    
    repeat = CCSprite::createWithSpriteFrameName("back1_5.png");
    repeat->setAnchorPoint(ccp(0,0));
    repeat->setPosition(ccp(0, 0));
    _background->addChild(repeat);
    
    repeat = CCSprite::createWithSpriteFrameName("back1_5.png");
    repeat->setAnchorPoint(ccp(0,0));
    repeat->setPosition(ccp(repeat->getContentSize().width - 1, 0));
    _background->addChild(repeat);
    
    repeat = CCSprite::createWithSpriteFrameName("back1_5.png");
    repeat->setAnchorPoint(ccp(0,0));
    repeat->setPosition(ccp(2 * (repeat->getContentSize().width - 1), 0));
    _background->addChild(repeat);
    CCSize _backgroundContentSize = repeat->getContentSize();
    _background->setContentSize(CCSizeMake(_backgroundContentSize.width*2, _backgroundContentSize.height*2));
    
    CCSprite* frontgroundSprite = CCSprite::createWithSpriteFrameName("back1_7.png");
    _frontground = CCSpriteBatchNode::createWithTexture(frontgroundSprite->getTexture(),3);
//    _frontground = CCSpriteBatchNode::create("back1_7.png");
    _frontground->setAnchorPoint(ccp(0,0));
    _frontground->setScale(2);
    _frontground->setPosition(ccp(0,0));
    addChild(_frontground, BackGround_Z);
    
    repeat = CCSprite::createWithSpriteFrameName("back1_7.png");
    repeat->setAnchorPoint(ccp(0,0));
    repeat->setPosition(ccp(0, 0));
    _frontground->addChild(repeat);
    
    repeat = CCSprite::createWithSpriteFrameName("back1_7.png");
    repeat->setAnchorPoint(ccp(0,0));
    repeat->setPosition(ccp(repeat->getContentSize().width - 1, 0));
    _frontground->addChild(repeat);
    CCSize _frontgroundContentSize = repeat->getContentSize();
    _frontground->setContentSize(CCSizeMake(_frontgroundContentSize.width*2, _frontgroundContentSize.height*2));
    
}

void CGameScene::initEffect()
{
    //特效
    CCArray*    pEffectFrames = CCArray::create();
    pEffectFrames->removeAllObjects();
    char str[128] = {0};
    
    if (CGameData::getInstance()->getEffectEnable())
    {
        //尘土
        for(int j = 1; j <15; j++)
        {
            sprintf(str, "EffectDust_%d.png", j);
            CCSpriteFrame* frame = CCSpriteFrameCache::sharedSpriteFrameCache()->spriteFrameByName(str);
            if(!frame) break;
            pEffectFrames->addObject(frame);
        }
        CCAnimation* animation1 = CCAnimation::createWithSpriteFrames(pEffectFrames,0.07);
        CCAnimationCache::sharedAnimationCache()->addAnimation(animation1, "EffectDust");
        //跳跃
        pEffectFrames->removeAllObjects();
        for(int j = 1; j <15; j++)
        {
            sprintf(str, "EffectJump_%d.png", j);
            CCSpriteFrame* frame = CCSpriteFrameCache::sharedSpriteFrameCache()->spriteFrameByName(str);
            if(!frame) break;
            pEffectFrames->addObject(frame);
        }
        CCAnimation* animation2 = CCAnimation::createWithSpriteFrames(pEffectFrames,0.07);
        CCAnimationCache::sharedAnimationCache()->addAnimation(animation2, "EffectJump");
        //连跳
        pEffectFrames->removeAllObjects();
        for(int j = 1; j <15; j++)
        {
            sprintf(str, "EffectDoubleJump_%d.png", j);
            CCSpriteFrame* frame = CCSpriteFrameCache::sharedSpriteFrameCache()->spriteFrameByName(str);
            if(!frame) break;
            pEffectFrames->addObject(frame);
        }
        CCAnimation* animation3 = CCAnimation::createWithSpriteFrames(pEffectFrames,0.07);
        CCAnimationCache::sharedAnimationCache()->addAnimation(animation3, "EffectDoubleJump");
    }
    //加速
    pEffectFrames->removeAllObjects();
    for(int j = 1; j <15; j++)
    {
        sprintf(str, "effect_speedUp_%d.png", j);
        CCSpriteFrame* frame = CCSpriteFrameCache::sharedSpriteFrameCache()->spriteFrameByName(str);
        if(!frame) break;
        pEffectFrames->addObject(frame);
    }
    CCAnimation* animation6 = CCAnimation::createWithSpriteFrames(pEffectFrames,0.07);
    CCAnimationCache::sharedAnimationCache()->addAnimation(animation6, "EffectSpeedUp");
    
    //复活
    pEffectFrames->removeAllObjects();
    for(int j = 1; j <15; j++)
    {
        sprintf(str, "effect_rebirth_%d.png", j);
        CCSpriteFrame* frame = CCSpriteFrameCache::sharedSpriteFrameCache()->spriteFrameByName(str);
        if(!frame) break;
        pEffectFrames->addObject(frame);
    }
    CCAnimation* aniReborn = CCAnimation::createWithSpriteFrames(pEffectFrames,0.07);
    CCAnimationCache::sharedAnimationCache()->addAnimation(aniReborn, "EffectRebirth");
    
    //倒计时
    pEffectFrames->removeAllObjects();
    for(int j = 1; j <15; j++)
    {
        sprintf(str, "Anim_CountDown_%d.png", j);
        CCSpriteFrame* frame = CCSpriteFrameCache::sharedSpriteFrameCache()->spriteFrameByName(str);
        if(!frame) break;
        pEffectFrames->addObject(frame);
    }
    CCAnimation* animation7 = CCAnimation::createWithSpriteFrames(pEffectFrames,0.1666);
    CCAnimationCache::sharedAnimationCache()->addAnimation(animation7, "CountDown");
    
    pEffectFrames->removeAllObjects();
    for(int j = 1; j <15; j++)
    {
        sprintf(str, "Anim_CountDown%d.png", j);
        CCSpriteFrame* frame = CCSpriteFrameCache::sharedSpriteFrameCache()->spriteFrameByName(str);
        if(!frame) break;
        pEffectFrames->addObject(frame);
    }
    CCAnimation* animation8 = CCAnimation::createWithSpriteFrames(pEffectFrames,1);
    CCAnimationCache::sharedAnimationCache()->addAnimation(animation8, "CountDownLetter");
}

void CGameScene::updateRunEffect(CCObject* pSender)
{
    if (m_pPlayer->getPlayerState() == kPlayerMoving)
    {
        CCSprite* pEffect = CCSprite::create("blankpic.png");
        pEffect->setAnchorPoint(ccp(1, 0));
        pEffect->setPosition(m_pPlayer->getPosition());
        m_pEnvironment->addChild(pEffect, Player_Z);
        
        
        CCAction* pAct = CCSequence::create(CCAnimate::create(CCAnimationCache::sharedAnimationCache()->animationByName("EffectDust")),
                                                              CCCallFuncN::create(this, callfuncN_selector(CGameScene::removeEffect)),
                                                              NULL);
        pEffect->runAction(pAct);
    }
    
}

void CGameScene::removeEffect(CCObject* pSender)
{
    m_pEnvironment->removeChild((CCSprite*)pSender);
}

void CGameScene::removeSprite(CCObject* pSender)
{
    this->removeChild((CCLayerColor*)pSender);
}

void CGameScene::initUI()
{
    CCSpriteFrameCache* cache = CCSpriteFrameCache::sharedSpriteFrameCache();

    m_pUILayer = CCLayer::create();
    m_pUILayer->setContentSize(CCSizeMake(1280, 720));
    m_pUILayer->setPosition(ccp(0,-20));
    m_pUILayer->setAnchorPoint(CCPointZero);
    m_pUILayer->setScale(1.1f);
    addChild(m_pUILayer, UI_Z);
    
    m_pItemTime = CCSprite::create();
    m_pItemTime->setAnchorPoint(ccp(0, 0));
    m_pItemTime->setPosition(ccp(550, _screenSize.height));
    m_pUILayer->addChild(m_pItemTime, UI_Z);
    
    CCSprite* ItemTime_BGSprite = CCSprite::createWithSpriteFrameName("UIGame_Effect_BG.png");
    ItemTime_BGSprite->setAnchorPoint(ccp(0, 0));
    ItemTime_BGSprite->setPosition(ccp(0, 0));
    m_pItemTime->addChild(ItemTime_BGSprite, UI_Z);
    
    m_pItemTimeProgress = CCProgressTimer::create(CCSprite::createWithSpriteFrameName("UIGame_EffectProgress_1.png"));
    m_pItemTimeProgress->setType(kCCProgressTimerTypeBar);
    m_pItemTimeProgress->setBarChangeRate(ccp(1.0,0));
    m_pItemTimeProgress->setMidpoint(ccp(0,0));
    m_pItemTimeProgress->setPercentage(0);
    m_pItemTimeProgress->setPosition(ccp(21, 30));
    m_pItemTimeProgress->setAnchorPoint(ccp(0, 0));
    m_pItemTime->addChild(m_pItemTimeProgress, UI_Z);
    

    
    m_pHeartArray = CCArray::create();
    m_pHeartArray->retain();
    for (int i=0; i<3; i++)
    {
        CCSprite* pHeartBGSprite = CCSprite::createWithSpriteFrameName("UIGame_HeartBG.png");
        pHeartBGSprite->setPosition(ccp(120 + i*30 , _screenSize.height - 100));
        m_pUILayer->addChild(pHeartBGSprite, UI_Z);
        
        CCSprite* pHeart = CCSprite::createWithSpriteFrameName("UIGame_Heart.png");
        pHeart->setPosition(ccp(120 + i*30 , _screenSize.height - 100));
        m_pUILayer->addChild(pHeart, UI_Z);
        m_pHeartArray->addObject(pHeart);
    }
    
    CCSprite* pVITBGSprite = CCSprite::createWithSpriteFrameName("UIGame_VITBG.png");
    pVITBGSprite->setPosition(ccp(280, _screenSize.height -100));
    m_pUILayer->addChild(pVITBGSprite, UI_Z);

    m_pVIT_Label = CCLabelTTF::create("100", "Arial-BoldMT",28,CCSizeMake(100, 48),kCCTextAlignmentCenter);
    m_pVIT_Label->setPosition(ccp(287,_screenSize.height - 107));
    m_pUILayer->addChild(m_pVIT_Label, UI_Z);
    
    CCSprite* pGoldBGSprite = CCSprite::createWithSpriteFrameName("UIGame_GoldBG.png");
    pGoldBGSprite->setPosition(ccp(910, _screenSize.height -100));
    m_pUILayer->addChild(pGoldBGSprite, UI_Z);

    m_pGoldLabel = CCLabelTTF::create("0", "Arial-BoldMT",28,CCSizeMake(100, 48),kCCTextAlignmentCenter);
    m_pGoldLabel->setPosition(ccp(922,_screenSize.height - 107));
    m_pUILayer->addChild(m_pGoldLabel, UI_Z);
    
    m_pBonusSprite = CCSprite::createWithSpriteFrameName("UIGame_Bonus.png");
    m_pBonusSprite->setPosition(ccp(752,_screenSize.height - 167));
    m_pUILayer->addChild(m_pBonusSprite, UI_Z);
    
    pBonusLabel = CCLabelAtlas::create("","number.png", 41, 39, '0');
    pBonusLabel->setPosition(ccp(822,_screenSize.height - 187));
    m_pUILayer->addChild(pBonusLabel, UI_Z);
    
    m_pDistanceSprite = CCSprite::createWithSpriteFrameName("UI_Distance.png");
    m_pDistanceSprite->setAnchorPoint(ccp(0, 0));
    m_pDistanceSprite->setPosition(ccp(370, _screenSize.height ));
//    m_pDistanceSprite->setPosition(ccp(370, _screenSize.height - 50));
    m_pUILayer->addChild(m_pDistanceSprite, UI_Z);
    
    m_pMoveDistanceLabel = CCLabelTTF::create("0", "Arial-BoldMT",40,CCSizeMake(400, 48),kCCTextAlignmentCenter);
    m_pMoveDistanceLabel->setPosition(ccp(80,30));
    m_pMoveDistanceLabel->setColor(ccc3(255, 255, 0));
    m_pDistanceSprite->addChild(m_pMoveDistanceLabel,UI_Z);
    
//    m_pMapIndexLabel = CCLabelTTF::create("", "Arial-BoldMT",40,CCSizeMake(400, 48),kCCTextAlignmentLeft);
//    m_pMapIndexLabel->setPosition(ccp(_screenSize.width - 200,_screenSize.height*0.5f - 100));
//    m_pMapIndexLabel->setColor(ccc3(255, 255, 0));
//    addChild(m_pMapIndexLabel,UI_Z);
    
    CCSprite* jumpItem = CCSprite::createWithSpriteFrameName("UIGame_Jump.png");
    jumpItem->setPosition(ccp(640+450, 120));
    addChild(jumpItem, UI_Z);
    
    CCSprite* slipItem = CCSprite::createWithSpriteFrameName("UIGame_Slip.png");
    slipItem->setPosition(ccp(640-450, 120));
    addChild(slipItem, UI_Z);
    
    CCMenuItemImage *pSettingItem = CCMenuItemImage::create();
    pSettingItem->setNormalSpriteFrame(cache->spriteFrameByName("UIGame_Pause.png"));
    pSettingItem->setSelectedSpriteFrame(cache->spriteFrameByName("UIGame_Pause.png"));
    pSettingItem->initWithTarget(this, menu_selector(CGameScene::menuPauseCallback) );
    pSettingItem->setPosition( ccp(1020,_screenSize.height-100) );
    
    CCMenu* pMenu = CCMenu::create(pSettingItem, NULL);
    pMenu->setAnchorPoint(CCPointZero);
    pMenu->setPosition( CCPointZero );
    m_pUILayer->addChild(pMenu, UI_Z);
    
    //复活倒计时
    m_pResurrectionLayer = CCLayer::create();
    m_pResurrectionLayer->setContentSize(CCSizeMake(1280, 720));
    m_pResurrectionLayer->setPosition(1280/2, 720/2);
    m_pResurrectionLayer->setAnchorPoint(ccp(0.5, 0.5));
    m_pResurrectionLayer->ignoreAnchorPointForPosition(true);
    addChild(m_pResurrectionLayer, UI_Z+1);
    m_pResurrectionLayer->setVisible(false);
    
    
    CCMenuItemImage *pResurrectionItem = CCMenuItemImage::create();
    pResurrectionItem->setNormalSpriteFrame(cache->spriteFrameByName("btn_score_Return.png"));
    pResurrectionItem->setSelectedSpriteFrame(cache->spriteFrameByName("btn_score_Return.png"));
    pResurrectionItem->initWithTarget(this, menu_selector(CGameScene::reborn) );
    pResurrectionItem->setPosition( ccp(0,0) );
    
    CCLabelTTF * continuelab = CCLabelTTF::create("Continue", "Arial-BoldMT",30,CCSizeMake(400, 48),kCCTextAlignmentCenter);
    continuelab->setPosition(ccp(130, 60));
    pResurrectionItem ->addChild(continuelab);
    
    //pResurrectionItem->setScale(0.5f);
    
    CCSprite* pResItem = CCSprite::createWithSpriteFrameName("UIGame_StartOver.png");
   
    pResItem->setPosition(ccp(0, 50));
    m_pResurrectionLayer->addChild(pResItem);
    
    CCMenu* pResurrectionMenu = CCMenu::create(pResurrectionItem, NULL);
    pResurrectionMenu->setContentSize(pResItem->getContentSize());
    pResurrectionMenu->setPosition( ccp(0, -180) );
    m_pResurrectionLayer->addChild(pResurrectionMenu, UI_Z);
    
    m_pResurrectionTimerSprite = CCSprite::createWithSpriteFrameName("Anim_CountDown_1.png");
    m_pResurrectionTimerSprite->setPosition(ccp(170, -180));
    m_pResurrectionLayer->addChild(m_pResurrectionTimerSprite, UI_Z);
    m_pResurrectionTimerSprite->setScale(0.5);
    
    m_pResurrectionTimeSprite = CCSprite::createWithSpriteFrameName("Anim_CountDown1.png");
    m_pResurrectionTimeSprite->setPosition(ccp(170, -180));
    m_pResurrectionLayer->addChild(m_pResurrectionTimeSprite, UI_Z);
    m_pResurrectionTimeSprite->setScale(0.5);
    
    //显示道具
    char szTexutre[128]={0};
    sprintf(szTexutre, "ui_main_Item%d.png", 12);
    CCSprite* pItem = CCSprite::createWithSpriteFrameName(szTexutre);
    pItem->setAnchorPoint(CCPointZero);
    pItem->setPosition(ccp(120, _screenSize.height - 170));
    addChild(pItem, UI_Z+100);
    
    CCSprite* AmountSprite = CCSprite::createWithSpriteFrameName("ui-main-RedDot.png");
    AmountSprite->setPosition(ccp(50, 15));
    AmountSprite->setScale(0.9f);
    pItem->addChild(AmountSprite);
    
    char szText[32]={0};
    sprintf(szText, "%d", CGameData::getInstance()->getMedicalKit());
    m_pMedicKitAmountLabel = CCLabelTTF::create(szText, "Arial-BoldMT", 24,CCSizeMake(85, 60),
                                        kCCTextAlignmentLeft,kCCVerticalTextAlignmentCenter);
    m_pMedicKitAmountLabel->setColor(ccYELLOW);
    m_pMedicKitAmountLabel->setPosition(ccp(58, 20 ));
    AmountSprite->addChild(m_pMedicKitAmountLabel);
    
    
    //在进度条
    progBackSp = CCSprite::createWithSpriteFrameName("ui-progback.png");
    progBackSp ->setPosition(ccp(_screenSize.width*0.5f, 60));
    addChild(progBackSp,UI_Z);
    
    m_pToSpaceProgress = CCProgressTimer::create(CCSprite::createWithSpriteFrameName("ui-progtop.png"));
    m_pToSpaceProgress->setType(kCCProgressTimerTypeBar);
    m_pToSpaceProgress->setBarChangeRate(ccp(1.0,0));
    m_pToSpaceProgress->setMidpoint(ccp(0,0));
    m_pToSpaceProgress->setPercentage(0);
    
    m_pToSpaceProgress->setPosition(ccp(62, 25));//62 25
    m_pToSpaceProgress->setAnchorPoint(ccp(0, 0));
    
    progBackSp->addChild(m_pToSpaceProgress);
    
    
    CCSprite * progIconSp = CCSprite::createWithSpriteFrameName("ui-progicon.png");
    progIconSp ->setPosition(ccp(36, 36));//62 25
    progBackSp->addChild(progIconSp);
    
    
//    onTakeABreak(NULL);
}

//void CGameScene::setViewpointCenter(CCPoint position)
//{
//    // 求出屏幕的范围包括宽和高
//    CCSize winSize = CCDirector::sharedDirector()->getWinSize();
//    //显示屏幕中心点的坐标大于屏幕宽和高的一半
//    int x = MAX(position.x, winSize.width / 2);
//    int y = MAX(position.y, winSize.height / 2);
//    
//    //求出的是整个瓦片地图的宽
//    //_tileMap->getMapSize().width瓦片地图横向有多少个瓦片
//    //_tileMap->getTileSize().width每一个瓦片的宽度
//    int mapWidth =m_pEnvironment->getTilemap_1()->getMapSize().width *m_pEnvironment->getTilemap_1()->getTileSize().width +
//                  m_pEnvironment->getTilemap_2()->getMapSize().width *m_pEnvironment->getTilemap_2()->getTileSize().width;
//    
//    
//    //求出的是整个瓦片地图的高
//    //_tileMap->getMapSize().height瓦片地图纵向有多少个瓦片
//    //_tileMap->getTileSize().height每一个瓦片的高度
//    int mapHeight =m_pEnvironment->getTilemap_1()->getMapSize().height *m_pEnvironment->getTilemap_1()->getTileSize().height;
//    
//    x = MIN(x, mapWidth- winSize.width / 2);
//    y = MIN(y, mapHeight - winSize.height / 2);
//    
//    //目标点
//    CCPoint actualPoint = ccp(x, y);
//    
//    //屏幕的中心点
//    CCPoint viewCenterPoint = ccp(winSize.width / 2,winSize.height / 2);
//    //计算出重置显示屏幕的中心点
//    //ccpSub 返回的是viewCenterPoint.x - actualPoint.x和viewCenterPoint.y - actualPoint.y
//    CCPoint viewPoint = ccpSub(viewCenterPoint, actualPoint);
//    //重置显示屏幕的中心点
//    this->setPosition(viewPoint);
//}

void CGameScene::onEnter()
{
    CCLayer::onEnter();
    
    CCNotificationCenter::sharedNotificationCenter()->addObserver(this,
                                                                  callfuncO_selector(CGameScene::updateHealthBar),
                                                                  "updateHealthBar",
                                                                  NULL);
    CCNotificationCenter::sharedNotificationCenter()->addObserver(this,
                                                                  callfuncO_selector(CGameScene::updateVITBar),
                                                                  "updateVITBar",
                                                                  NULL);
    CCNotificationCenter::sharedNotificationCenter()->addObserver(this,
                                                                  callfuncO_selector(CGameScene::updateGoldeBar),
                                                                  "updateGoldeBar",
                                                                  NULL);
    CCNotificationCenter::sharedNotificationCenter()->addObserver(this,
                                                                  callfuncO_selector(CGameScene::updateMoveDistance),
                                                                  "updateMoveDistance",
                                                                  NULL);
    
    CCNotificationCenter::sharedNotificationCenter()->addObserver(this,
                                                                  callfuncO_selector(CGameScene::onAccelerateCountDown),
                                                                  "accelerateCountDown",
                                                                  NULL);
    CCNotificationCenter::sharedNotificationCenter()->addObserver(this,
                                                                  callfuncO_selector(CGameScene::onAbsortCountDown),
                                                                  "absortCountDown",
                                                                  NULL);
    CCNotificationCenter::sharedNotificationCenter()->addObserver(this,
                                                                  callfuncO_selector(CGameScene::onGaintCountDown),
                                                                  "giantCountDown",
                                                                  NULL);
    CCNotificationCenter::sharedNotificationCenter()->addObserver(this,
                                                                  callfuncO_selector(CGameScene::popUpReborn),
                                                                  "playerDie",
                                                                  NULL);
    
//    CCNotificationCenter::sharedNotificationCenter()->addObserver(this, callfuncO_selector(CGameScene::updateMapIndex), "updateMapIndex", NULL);
    CCNotificationCenter::sharedNotificationCenter()->addObserver(this, callfuncO_selector(CGameScene::onPortalCall), "Portal", NULL);
    CCNotificationCenter::sharedNotificationCenter()->addObserver(this, callfuncO_selector(CGameScene::onResumeGame), "resumeGame", NULL);
    CCNotificationCenter::sharedNotificationCenter()->addObserver(this, callfuncO_selector(CGameScene::onBonusMap), "bonusMap", NULL);
    CCNotificationCenter::sharedNotificationCenter()->addObserver(this, callfuncO_selector(CGameScene::onCompleteBonusMap), "completeBonusMap", NULL);
    
    CCNotificationCenter::sharedNotificationCenter()->addObserver(this, callfuncO_selector(CGameScene::onTakeABreak), "TakeABreak", NULL);
    CCNotificationCenter::sharedNotificationCenter()->addObserver(this, callfuncO_selector(CGameScene::addAlert), "AddNoMoneyAlert", NULL);
}

void CGameScene::onExit()
{
    CCLayer::onExit();
    CCNotificationCenter::sharedNotificationCenter()->removeObserver(this, "updateHealthBar");
    CCNotificationCenter::sharedNotificationCenter()->removeObserver(this, "updateVITBar");
    CCNotificationCenter::sharedNotificationCenter()->removeObserver(this, "updateGoldeBar");
    CCNotificationCenter::sharedNotificationCenter()->removeObserver(this, "updateMoveDistance");
    
    CCNotificationCenter::sharedNotificationCenter()->removeObserver(this, "accelerateCountDown");
    CCNotificationCenter::sharedNotificationCenter()->removeObserver(this, "absortCountDown");
    CCNotificationCenter::sharedNotificationCenter()->removeObserver(this, "giantCountDown");
    
    CCNotificationCenter::sharedNotificationCenter()->removeObserver(this, "playerDie");
    CCNotificationCenter::sharedNotificationCenter()->removeObserver(this, "updateMapIndex");
    
    CCNotificationCenter::sharedNotificationCenter()->removeObserver(this, "Portal");
    CCNotificationCenter::sharedNotificationCenter()->removeObserver(this, "resumeGame");
    CCNotificationCenter::sharedNotificationCenter()->removeObserver(this, "bonusMap");
    CCNotificationCenter::sharedNotificationCenter()->removeObserver(this, "completeBonusMap");
    CCNotificationCenter::sharedNotificationCenter()->removeObserver(this, "TakeABreak");

    CCNotificationCenter::sharedNotificationCenter()->removeObserver(this, "AddNoMoneyAlert");
    CCDirector::sharedDirector()->getTouchDispatcher()->removeDelegate(this);
}

void CGameScene::registerWithTouchDispatcher(void)
{
//    CCDirector::sharedDirector()->getTouchDispatcher()->addTargetedDelegate(this, this->getTouchPriority(), true);
    cocos2d::CCDirector::sharedDirector()->getTouchDispatcher()->addStandardDelegate(this,0);
}

void CGameScene::ccTouchesBegan(CCSet *pTouches, CCEvent *pEvent)
{
    CCSetIterator it = pTouches->begin();
	CCTouch* touch = (CCTouch*)(*it);
	CCPoint touchpoint = touch->getLocationInView();
	touchpoint = CCDirector::sharedDirector()->convertToGL( touchpoint );
    int touchCount = pTouches->count();
    
    if(touchCount == 1)
    {
        if (touchpoint.x < _screenSize.width * 0.5f)
        {
            slip();
        }
        else
        {
            jump();
        }
    }
    else if (touchCount >= 2)
    {
        CCSetIterator iter2 = pTouches->end();
        iter2--;
        CCTouch *touch2 = (CCTouch*)(*(iter2));
        touchpoint = touch2->getLocationInView();
        touchpoint = CCDirector::sharedDirector()->convertToGL(touchpoint);
        
        if (touchpoint.x < _screenSize.width * 0.5f)
        {
            slip();
        }
        else
        {
            jump();
        }
    }
    
    
}

void CGameScene::ccTouchesEnded(CCSet *pTouches, CCEvent *pEvent)
{
    if (m_pPlayer->getSliping()) {
        m_pPlayer->recoverFromSlip();
    }
    readyToSlip =false;
}

void CGameScene::update (float dt)
{
    if (!m_pPlayer->getDying() && !m_pPlayer->getResting())
    {
        m_pPlayer->update(dt);
        m_pEnvironment->move(m_pPlayer);
        m_pPlayer->place();
        updateBackGround(m_pPlayer->getVector().x);
    }
    if (m_pPlayer->getPositionX() + _screenSize.width*0.3f < m_pEnvironment->getMoveDistance())
    {
        m_pPlayer->setDying(true);
        popUpReborn();        
    }
    updateToSpaceCountDown();
    updateAccelerateCountDown();
    
    if (_bChangeMap)
    {
        int iStage = (m_pEnvironment->getStage())%2 +1;
        int iMapID = m_pEnvironment->getMapIndex();
        
        changeMap(iStage, iMapID);
        
        _bChangeMap = false;
    }
    
    if (_bBonusMap)
    {
        progBackSp->setVisible(false);
        
        currentStage = m_pEnvironment->getStage();
        currentMap = m_pEnvironment->getMapIndex();
        
        changeMap(3, 1);
        
        _bBonusMap = false;
    }
    if (_bCompleteBonusMap)
    {
//        int iStage =  int(CCRANDOM_0_1()*100)%2 + 1;
        int iStage = currentStage;
//        int iMapIndex =  int(CCRANDOM_0_1()*100)%80 + 1;
        int iMapIndex = currentMap;
        changeMap(iStage, iMapIndex-1);
        
        _bCompleteBonusMap = false;
        
        
        progBackSp->setVisible(true);
    }
    
    if (readyToSlip) {
        readyToSlip = false;
        if (m_pPlayer->getPlayerAction() != kActionSliping) {
            slip();
        }
        
    }
}

void CGameScene::updateBackGround(float xMove)
{
    if (xMove > 0)
    {
		//一直往左移
        if (m_pEnvironment->getStage() == 3) {
            _frontground->setPositionX(_frontground->getPosition().x - xMove * 1.25f);
        }
        else
        {
            _frontground->setPositionX(_frontground->getPosition().x - xMove * 0.7f);
        }
        _background->setPositionX(_background->getPosition().x - xMove * 0.25f);
        _clound->setPositionX(_clound->getPosition().x - xMove * 0.15f);
        
        
        //移完一个宽度时，重新把位置设置为接近0的位置
        if (_frontground->getPositionX() <= -_frontground->getContentSize().width)
        {
            _frontground->setPositionX(0);
        }
        if (_background->getPositionX() <= -_background->getContentSize().width) {
            _background->setPositionX(0);
        }
        if (_clound->getPositionX() < -_clound->getContentSize().width) {
            _clound->setPositionX(0);
        }
    }
}

void CGameScene::changeMap(int iStage, int iIndex)
{
    CCLayerColor* pBlinkLayer = CCLayerColor::create(ccc4(255, 255, 255, 255), 1280, 720);
    addChild(pBlinkLayer, UI_Z);
    pBlinkLayer->runAction( CCSequence::create(CCFadeIn::create(0.25),
                                               CCFadeOut::create(0.5),
                                               CCCallFuncN::create(this, callfuncN_selector(CGameScene::removeSprite)),
                                               NULL));
    
    changeBackGround(iStage);
    removeChild(m_pEnvironment, true);
    CCLog("changmap stage = %d index %d",iStage,iIndex);
    m_pEnvironment = CEnvironment::create(iStage,iIndex);
    addChild(m_pEnvironment, Terrain_Z);
    m_pEnvironment->setPosition(ccp(0, 0));
    m_pEnvironment->setMoveDistance(0);
    this->setPosition(0, 0);
    
    m_pPlayer = CPlayer::createPlayer("Character1_");
    m_pPlayer->setData("Character1", 1, 3, 0.025, 10);
    CCPoint vBornPosition = ccp(200,640);
    m_pPlayer->setPosition(vBornPosition);
    m_pPlayer->setNextPosition(ccp(vBornPosition.x, 640));
    m_pPlayer->setVector ( ccp(m_pPlayer->getVector().x, 0) );
    m_pPlayer->setAnchorPoint(ccp(0.5f,0.4f));
    m_pPlayer->playAction("Fall_",false);
    
    m_pEnvironment->addChild(m_pPlayer,Player_Z);
}

void CGameScene::changeBackGround(int iStage)
{
    _clound->removeAllChildren();
    _background->removeAllChildren();
    _frontground->removeAllChildren();
    
    removeChild(_clound,true);
    removeChild(_background,true);
    removeChild(_frontground,true);
    
    _iBGIndex = iStage;
    
    CCSprite * repeat;
    char szTexutre[64]={0};
    sprintf(szTexutre, "back%d_4.jpg",_iBGIndex);
    _clound = CCSpriteBatchNode::create(szTexutre);
    _clound->setAnchorPoint(ccp(0,0));
    _clound->setScale(2);
    _clound->setPosition(CCPointZero);
    addChild(_clound, BackGround_Z);
    
    repeat = CCSprite::create(szTexutre);
    repeat->setAnchorPoint(ccp(0,0));
    repeat->setPosition(ccp(0, 0));
    _clound->addChild(repeat);
    
    repeat = CCSprite::create(szTexutre);
    repeat->setAnchorPoint(ccp(0,0));
    repeat->setPosition(ccp(repeat->getContentSize().width - 1, 0));
    _clound->addChild(repeat);
    
    if (iStage == 2) {
        repeat = CCSprite::create(szTexutre);
        repeat->setAnchorPoint(ccp(0,0));
        repeat->setPosition(ccp(2*(repeat->getContentSize().width) - 1, 0));
        _clound->addChild(repeat);
    }
    
    CCSize _cloundContentSize = repeat->getContentSize();
    _clound->setContentSize(CCSizeMake(_cloundContentSize.width*2, _cloundContentSize.height*2));
    
    sprintf(szTexutre, "back%d_5.png",_iBGIndex);
    CCSprite* backgroundSprite = CCSprite::createWithSpriteFrameName(szTexutre);
    _background = CCSpriteBatchNode::createWithTexture(backgroundSprite->getTexture(),3);
//    sprintf(szTexutre, "back%d_5.png",_iBGIndex);
//    _background = CCSpriteBatchNode::create(szTexutre);
    _background->setAnchorPoint(ccp(0,0));
    _background->setScale(2);
    _background->setPosition(CCPointZero);
    addChild(_background, BackGround_Z);
    
    repeat = CCSprite::createWithSpriteFrameName(szTexutre);
    repeat->setAnchorPoint(ccp(0,0));
    repeat->setPosition(ccp(0, 0));
    _background->addChild(repeat);
    
    repeat = CCSprite::createWithSpriteFrameName(szTexutre);
    repeat->setAnchorPoint(ccp(0,0));
    repeat->setPosition(ccp(repeat->getContentSize().width - 1, 0));
    _background->addChild(repeat);
    
    repeat = CCSprite::createWithSpriteFrameName(szTexutre);
    repeat->setPosition(ccp(2 * (repeat->getContentSize().width - 1), 0));
    _background->addChild(repeat);
    _background->setContentSize(repeat->getContentSize());
    
    CCSize _backgroundContentSize = repeat->getContentSize();
    _background->setContentSize(CCSizeMake(_backgroundContentSize.width*2, _backgroundContentSize.height*2));
    
    sprintf(szTexutre, "back%d_7.png",_iBGIndex);
    CCSprite* frontgroundSprite = CCSprite::createWithSpriteFrameName(szTexutre);
    _frontground = CCSpriteBatchNode::createWithTexture(frontgroundSprite->getTexture(),3);
//    sprintf(szTexutre, "back%d_7.png",_iBGIndex);
//    _frontground = CCSpriteBatchNode::create(szTexutre);
    _frontground->setAnchorPoint(ccp(0,0));
    _frontground->setScale(2);
    _frontground->setPosition(ccp(0,0));
    addChild(_frontground, BackGround_Z);
    
    repeat = CCSprite::createWithSpriteFrameName(szTexutre);
    repeat->setAnchorPoint(ccp(0,0));
    repeat->setPosition(ccp(0, 0));
    _frontground->addChild(repeat);
    
    repeat = CCSprite::createWithSpriteFrameName(szTexutre);
    repeat->setAnchorPoint(ccp(0,0));
    repeat->setPosition(ccp(repeat->getContentSize().width - 1, 0));
    _frontground->addChild(repeat);
    
    if (iStage == 2) {
        repeat = CCSprite::createWithSpriteFrameName(szTexutre);
        repeat->setAnchorPoint(ccp(0,0));
        repeat->setPosition(ccp(2*(repeat->getContentSize().width) - 1, 0));
        _frontground->addChild(repeat);
    }
    
    CCSize _frontgroundContentSize = repeat->getContentSize();
    _frontground->setContentSize(CCSizeMake(_frontgroundContentSize.width*2, _frontgroundContentSize.height*2));
    
    _background->setPositionX(0);
    _clound->setPositionX(0);
}

void CGameScene::updateHealthBar(CCObject* pSender)
{
    int iHP = ((CCInteger*)pSender)->getValue();

    for (int i=0; i<m_pHeartArray->count(); i++)
    {
        CCSprite* pHeart = (CCSprite*)m_pHeartArray->objectAtIndex(i);
        if (iHP > i) {
            pHeart->setVisible(true);
        }
        else{
            pHeart->setVisible(false);
        }
    }
}

void CGameScene::updateVITBar(CCObject* pSender)
{
    int iVIT = ((CCInteger*)pSender)->getValue();

    char szVIT[32]={0};
    sprintf(szVIT, "%d",iVIT);
    m_pVIT_Label->setString(szVIT);
}

void CGameScene::updateGoldeBar(CCObject* pSender)
{
    m_iGold += ((CCInteger*)pSender)->getValue();
    char szGold[128]={0};
    sprintf(szGold, "%d",m_iGold);
    m_pGoldLabel->setString(szGold);
    
    if (m_pEnvironment->getPartner() && m_pEnvironment->getPartner()->getAttatch()) {
        m_iBonusGold += ((CCInteger*)pSender)->getValue();
        sprintf(szGold, ":%.0f",m_iBonusGold*0.5f);
        pBonusLabel->setString(szGold);
        m_pBonusSprite->setVisible(true);
        pBonusLabel->setVisible(true);
    }
    else
    {
        m_pBonusSprite->setVisible(false);
        pBonusLabel->setVisible(false);
        m_iBonusGold = 0;
    }
    
}
   
void CGameScene::updateMoveDistance(CCObject* pSender)
{
    float freachDistance = m_fMoveDistance + ((CCFloat*)pSender)->getValue();
    
    if ((int)m_fMoveDistance == (int)freachDistance) {
        m_fMoveDistance = freachDistance;
        return;
    }
    m_fMoveDistance = freachDistance;
    
    int iMoveDistance = (int)m_fMoveDistance;
    char szMoveDistance[128]={0};
    sprintf(szMoveDistance, "%d",iMoveDistance);
    m_pMoveDistanceLabel->setString(szMoveDistance);
    
    if ( !m_bPopUpDistance && iMoveDistance >= 50 && iMoveDistance%50==0)
    {
        m_bPopUpDistance = true;
        CCAction* pAction = CCSequence::create(CCMoveTo::create(0.5, ccp(370, _screenSize.height-130)),
                                               CCDelayTime::create(3),
                                               CCMoveTo::create(0.5, ccp(370, _screenSize.height)),
                                               CCCallFuncN::create(this, callfuncN_selector(CGameScene::completePopUpDistance)),
                                               NULL);
        m_pDistanceSprite->runAction(pAction);
    }
    
        
    newHandAchieveCheck(iMoveDistance);
    
    
}

void CGameScene::onTakeABreak(CCObject* pSender)
{
    CCLayerColor * adverbg = CCLayerColor::create(ccc4(3, 2, 1, 150), 1280, 720);
    
    CCSprite* pAdverSprite =  CCSprite::createWithSpriteFrameName("UIGame_Advertisement.png");
    pAdverSprite ->setScale(1.4);
    adverbg->setTag(11088);
    this->addChild(adverbg,UI_Z+10);
    
    if (pAdverSprite)
    {
        pAdverSprite->setAnchorPoint(ccp(0.5f,0.5f));
        pAdverSprite->setPosition(ccp(1280/2, 720/2));
        
        pAdverSprite->setTag(11088);
        adverbg->addChild(pAdverSprite,UI_Z+10);
        
        char szText[128]={0};
        int iAdvertismentIndex = (CCRANDOM_0_1()*100);
        iAdvertismentIndex = iAdvertismentIndex%3;
        switch (iAdvertismentIndex) {
            case 0:
                sprintf(szText, "UC Browser has up to 130%% faster Facebook speed");
                break;
            case 1:
                sprintf(szText, "UC Browser gives you 40%% faster downloads");
                break;
            case 2:
                sprintf(szText, "UC Browser gives you 20%% faster browsing");
                break;
            default:
                break;
        }
        
        CCLabelTTF* pAdverLabel = CCLabelTTF::create(szText, "Arial-BoldMT",34,CCSizeMake(900, 100),kCCTextAlignmentCenter,kCCVerticalTextAlignmentCenter);
        pAdverLabel->setPosition(ccp(0, -50));
        pAdverLabel->setTag(11089);
        pAdverSprite->addChild(pAdverLabel,1);
        
        pAdverSprite->runAction(CCSequence::create(CCFadeIn::create(0.5f),CCDelayTime::create(2.5f),CCFadeOut::create(0.5f),
                                                   CCCallFuncN::create(this, callfuncN_selector(CGameScene::onFinishBreak)),NULL));
    }
    
    
}
void CGameScene::onFinishBreak()
{
    removeChildByTag(11088);
}

void CGameScene::onAccelerateCountDown(CCObject* pSender)
{
    CCDictionary* pDic = (CCDictionary*)pSender;
    m_iCountDown = ((CCInteger*)pDic->objectForKey("Duration"))->getValue();
    m_iCountDownTotal = ((CCInteger*)pDic->objectForKey("TotalDuration"))->getValue();
//    m_iCountDown = ((CCInteger*)pSender)->getValue();
    
    m_pItemTimeProgress->setPercentage(100);
    
    m_pItemTime->runAction(CCMoveTo::create(0.5, ccp(550, _screenSize.height-150)));
    
    //加入加速的图标
    CCSprite * accSp = CCSprite::createWithSpriteFrameName("ui_main_Item12.png");
    m_pItemTime ->addChild(accSp , 2001,2001);
    accSp ->setPosition(ccp(236, 45));
    
}
void CGameScene::onAbsortCountDown(CCObject * pSender)
{
    CCDictionary* pDic = (CCDictionary*)pSender;
    m_iCountDown = ((CCInteger*)pDic->objectForKey("Duration"))->getValue();
    m_iCountDownTotal = ((CCInteger*)pDic->objectForKey("TotalDuration"))->getValue();
    //    m_iCountDown = ((CCInteger*)pSender)->getValue();
    m_pItemTimeProgress->setPercentage(100);
    m_pItemTime->runAction(CCMoveTo::create(0.5, ccp(550, _screenSize.height-150)));
    
    //加入加速的图标
    CCSprite * accSp = CCSprite::createWithSpriteFrameName("ui_main_Item11.png");
    m_pItemTime ->addChild(accSp , 2001,2001);
    accSp ->setPosition(ccp(236, 45));
}

void CGameScene::onGaintCountDown(CCObject * pSender)
{
    CCDictionary* pDic = (CCDictionary*)pSender;
    m_iCountDown = ((CCInteger*)pDic->objectForKey("Duration"))->getValue();
    m_iCountDownTotal = ((CCInteger*)pDic->objectForKey("TotalDuration"))->getValue();
    //    m_iCountDown = ((CCInteger*)pSender)->getValue();
    m_pItemTimeProgress->setPercentage(100);
    m_pItemTime->runAction(CCMoveTo::create(0.5, ccp(550, _screenSize.height-150)));
    
    //加入加速的图标
    CCSprite * accSp = CCSprite::createWithSpriteFrameName("ui_main_Item13.png");
    m_pItemTime ->addChild(accSp , 2001,2001);
    accSp ->setPosition(ccp(236, 45));
}

void CGameScene::updateAccelerateCountDown()
{
    if(m_iCountDown >0)
    {
        m_iCountDown -= 10;
        
        float fPercentage = (m_iCountDown/(float)m_iCountDownTotal) * 100;
        m_pItemTimeProgress->setPercentage(fPercentage);
        
        if (m_iCountDown <=0)
        {
            m_iCountDown = 0;
            m_pItemTime->runAction(CCMoveTo::create(0.5, ccp(550, _screenSize.height+20)));
        }
    }
}

void CGameScene::updateToSpaceCountDown()
{
    
    
    int icurrentStage = m_pEnvironment->getStage();
    
    int ipass = (int)m_pEnvironment->getMoveDistance();
    CCLog("icurrentStage = %d  ipass =%d",icurrentStage,ipass );
    if(icurrentStage < 3){
        int icurrentMap = m_pEnvironment->getMapIndex();
        
        
//        float perc = ( ipass )*100/(60 * 32 * 10 );
////        CCLog("icurrentMap %d,ipass = %  dperc =%f", icurrentMap,ipass,perc );
////        if (icurrentMap < 12) {
//        if (icurrentMap < 22) {
//            
////            float perc = ( ipass )*100/(60 * 32 * 11 );
//            float perc = ( ipass )*100/(60 * 32 * 21 );
//            m_pToSpaceProgress->setPercentage(perc);
//        }else{
////            float perc = ( ipass - ((icurrentMap/10) *10 ) * (60 * 32 ) )*100/(60 * 32 * 10 );
////            CCLog("%dperc =%f", ipass,perc );
//            //因为回来后 ipass重新 从0计算。。。
//            float perc = ( ipass )*100/(60 * 32 * 10 );
//            m_pToSpaceProgress->setPercentage(perc);
//        }
        
        float perc = ( ipass )*100/(60 * 32 * 30 );
        m_pToSpaceProgress->setPercentage(perc);
        
    }else{
        
    }
    
    
}

//void CGameScene::onAbsorbCountDown(CCObject* pSender)
//{
//    
//}
//
//void CGameScene::updateAbsorbCountDown()
//{
//    
//}
//
//void CGameScene::onGiantCountDown(CCObject* pSender)
//{
//    
//}
//
//void CGameScene::updateGiantCountDown()
//{
//    
//}

void CGameScene::updateMapIndex(CCObject* pSender)
{
    char szText[64]={0};
    int iIndex = ((CCInteger*)pSender)->getValue();
    sprintf(szText, "MapIndex:%d",iIndex);
    m_pMapIndexLabel->setString(szText);
}

void CGameScene::onPortalCall(CCObject* pSender)
{
    _bChangeMap = true;
}

void CGameScene::onBonusMap(CCObject* pSender)
{
    _bBonusMap = true;
}

void CGameScene::onCompleteBonusMap(CCObject* pSender)
{
    _bCompleteBonusMap = true;
}

void CGameScene::completePopUpDistance(CCObject* pSender)
{
    m_bPopUpDistance = false;
}

void CGameScene::onResumeGame(CCObject* pSender)
{
    CCDirector::sharedDirector()->resume();
}


void CGameScene::jump()
{
    if (isDoingNewHand1 == 1) {
        isDoingNewHand1 = 2;
        removeNewHand();
    }
    if (isDoingNewHand2 == 1) {
        return;
    }
    if (m_pPlayer->getDying() || m_pPlayer->getIsAccelerate()) {
        return;
    }
    
    if (m_pPlayer->getPlayerState() == kPlayerFalling && !m_pPlayer->getDoubleJumping())
    {
        m_pPlayer->setDoubleJumping(true);
        m_pPlayer->setJumping(false);
        m_pPlayer->setPlayerState(kPlayerFly);
        m_pPlayer->stopActionByTag(1);
        
        m_pPlayer->doDoubleJump();
        
        CCSprite* pEffect = CCSprite::create("blankpic.png");
        pEffect->setAnchorPoint(ccp(0.7, 0.5));
        pEffect->setPosition(m_pPlayer->getPosition());
        m_pEnvironment->addChild(pEffect, Player_Z);
        
        if (CGameData::getInstance()->getEffectEnable())
        {
            CCAction* pAct2 = CCSequence::create(CCAnimate::create(CCAnimationCache::sharedAnimationCache()->animationByName("EffectDoubleJump")),
                                                 CCCallFuncN::create(this, callfuncN_selector(CGameScene::removeEffect)),
                                                 NULL);
            pEffect->runAction(pAct2);
        }
        CSoundManager::getInstance()->playSound("sound/role_jump_big.ogg");
        
    }
    else if (m_pPlayer->getPlayerState() == kPlayerMoving)
    {
        m_pPlayer->setJumping(true);
        m_pPlayer->stopActionByTag(1);
        
        m_pPlayer->doJump();
        
        CCSprite* pEffect = CCSprite::create("blankpic.png");
        pEffect->setAnchorPoint(ccp(0.7, 0));
        pEffect->setPosition(m_pPlayer->getPosition());
        m_pEnvironment->addChild(pEffect, Player_Z);
        
        if (CGameData::getInstance()->getEffectEnable())
        {
            CCAction* pAct2 = CCSequence::create(CCAnimate::create(CCAnimationCache::sharedAnimationCache()->animationByName("EffectJump")),
                                                 CCCallFuncN::create(this, callfuncN_selector(CGameScene::removeEffect)),
                                                 NULL);
            pEffect->runAction(pAct2);
        }
        CSoundManager::getInstance()->playSound("sound/role_jump.ogg");
    }
}

void CGameScene::slip()
{
    if (isDoingNewHand2 == 1) {
        isDoingNewHand2 = 2;
        removeNewHand();
    }
    if (isDoingNewHand1 == 1) {
        return;
    }
    if (m_pPlayer->getDying() || m_pPlayer->getIsAccelerate()) {
        return;
    }
   
    if (m_pPlayer->getPlayerState() != kPlayerFalling)
    {
        m_pPlayer->doSlip();
    }
     readyToSlip = true;
}

void CGameScene::menuPauseCallback(CCObject* pSender)
{
    CSoundManager::getInstance()->playSound("sound/ui_button.ogg");
    
    CGameData::getInstance()->setRunScore(m_iGold);
    CGameData::getInstance()->setRunDistance(m_fMoveDistance);
    CGameData::getInstance()->setGainGold(100 + m_fMoveDistance + m_iGold/100);
    
    CPauseLayer* pPauseLayer = CPauseLayer::create();
    pPauseLayer->setPosition(ccp(640-225,360-132.5));
    pPauseLayer->setScale(1.1f);
    this->addChild(pPauseLayer,600);
    
    CCDirector::sharedDirector()->pause();
}

void CGameScene::startGameCountDown()
{
    
}

void CGameScene::popUpReborn()
{
    if ( CGameData::getInstance()->getMedicalKit() > 0 && m_iRebornChance > 0)
    {
        m_pResurrectionLayer->setVisible(true);
        
        m_pResurrectionTimerSprite->runAction(CCRepeat::create(CCAnimate::create(CCAnimationCache::sharedAnimationCache()->animationByName("CountDown")), 3) );
        
        CCAction* pAct2 = CCSequence::create(CCAnimate::create(CCAnimationCache::sharedAnimationCache()->animationByName("CountDownLetter")),
                                             CCCallFuncN::create(this, callfuncN_selector(CGameScene::back2MainView)),
                                             NULL);
        m_pResurrectionTimeSprite->runAction(pAct2);
        
        if (CGameData::getInstance()->getEffectEnable())
        {
            unschedule(schedule_selector(CGameScene::updateRunEffect));
        }
        this->unscheduleUpdate();
    }
    else
    {
//        CGameData::getInstance()->setRunScore(m_iGold);
//        CGameData::getInstance()->setRunDistance(m_fMoveDistance);
//        CGameData::getInstance()->setGainGold(100 + m_fMoveDistance + m_iGold/100);
//        
//        CCScene* scene = CCScene::create();
//        CScoreScene* layer = CScoreScene::create();
//        scene->addChild(layer);
//        CCDirector::sharedDirector()->replaceScene(scene);
        
        CCAction* pAct2 = CCSequence::create(CCDelayTime::create(1),
                                             CCCallFuncN::create(this, callfuncN_selector(CGameScene::back2MainView)),
                                             NULL);
        m_pResurrectionTimeSprite->runAction(pAct2);
    }
    
}

void CGameScene::reborn(CCObject* pSender)
{
    if (m_iRebornChance >0)
    {
        m_pResurrectionLayer->setVisible(false);
        
        m_pPlayer->setHP(m_pPlayer->getMaxHP());
        m_pPlayer->setVIT(m_pPlayer->getMaxVIT());
        m_pPlayer->setDying(false);
        m_pPlayer->getItem(1);
        m_pPlayer->playAction("Fall_",false);
        m_pPlayer->setNextPosition(ccp(m_pPlayer->getNextPosition().x, 450));
        m_pPlayer->setVector ( ccp(m_pPlayer->getVector().x, 0) );
        
        m_pResurrectionTimerSprite->stopAllActions();
        m_pResurrectionTimeSprite->stopAllActions();
        
        updateHealthBar(CCInteger::create(m_pPlayer->getMaxHP()));
        updateVITBar(CCInteger::create(m_pPlayer->getMaxVIT()));
        
        if (CGameData::getInstance()->getEffectEnable())
        {
            this->schedule(schedule_selector(CGameScene::updateRunEffect), 1.5);
        }
        scheduleUpdate();
        
        m_iRebornChance--;
        CGameData::getInstance()->setMedicalKit(CGameData::getInstance()->getMedicalKit() -1);
        CGameData::getInstance()->updateData();
        
        char szText[32]={0};
        sprintf(szText, "x %d", CGameData::getInstance()->getMedicalKit());
        m_pMedicKitAmountLabel->setString(szText);
    }
    
}

void CGameScene::back2MainView(CCObject* pSender)
{
    CGameData::getInstance()->setRunScore(m_iGold);
    CGameData::getInstance()->setRunDistance(m_fMoveDistance);
    CGameData::getInstance()->setGainGold(100 + m_fMoveDistance + m_iGold/100);
    
    CCScene* scene = CCScene::create();
    CScoreScene* layer = CScoreScene::create();
    scene->addChild(layer);
    CCDirector::sharedDirector()->replaceScene(scene);
}


void CGameScene::newHandAchieveCheck(int _moveDist)
{
    CCLog("%d",_moveDist);
    if (!GameModel::getInstance()->hasDoNewHand) {
    
        if (isDoingNewHand1 ==0 || isDoingNewHand2 ==0) {
            
            if (_moveDist == 4 && isDoingNewHand1==0) {
                isDoingNewHand1 = 1;
                
                CCDirector::sharedDirector()->pause();
                
                CCSprite * newHandSp = CCSprite::createWithSpriteFrameName("newhand_aim.png");
                newHandSp->setAnchorPoint(ccp(0.5, 0));
                newHandSp->setScale(2.0);
                newHandSp->setPosition(ccp(640+450, 200));
                addChild(newHandSp, UI_Z , 10086);
            }
            if (_moveDist == 8 && isDoingNewHand2==0) {
                isDoingNewHand2 = 1;
                
                CCDirector::sharedDirector()->pause();
                
                CCSprite * newHandSp = CCSprite::createWithSpriteFrameName("newhand_aim.png");
                newHandSp->setScale(2.0);
                newHandSp->setAnchorPoint(ccp(0.5, 0));
                newHandSp->setPosition(ccp(640-450, 200));
                addChild(newHandSp, UI_Z, 10086);
                
                GameModel::getInstance()->hasDoNewHandOK();
            }
        }
    }
    
    GameModel::getInstance()->checkAchieve((int)m_fMoveDistance, m_iGold);
}

void CGameScene::removeNewHand()
{
    
    CCDirector::sharedDirector()->resume();

    CCSprite * newHandSp = (CCSprite *)getChildByTag(10086);
    removeChild(newHandSp);
}


void  CGameScene::addAlert(CCObject* pSender)
{

	CCScene * rscene = CCDirector::sharedDirector()->getRunningScene();
//	if(rscene->getChildByTag(212121))
//	{
//		return;
//	}
	    	CCLabelTTF * noMoneyAlert = CCLabelTTF::create("You dont have enough money,go to buy some!! ","STHeiti TC",35);
	    	noMoneyAlert->setPosition(ccp(1280/2 , 720/2));

	    	CCLabelTTF * noMoneyAlert2 = CCLabelTTF::create("You dont have enough money,go to buy some!! ","STHeiti TC",35);
	    	noMoneyAlert2->setColor(ccBLACK);
	    	  noMoneyAlert2->setPosition(ccp(1280/2 +2 , 720/2 -2));
	    	int alertTag = 212121;

	    	this->addChild(noMoneyAlert,alertTag + 100001 , 212121);
	    	this->addChild(noMoneyAlert2,alertTag + 100000 , 212121+1);

	    	CCDelayTime * deleyAct = CCDelayTime::create(2.0);
	    	CCCallFunc * callsa = CCCallFunc::create((CCObject*)this,callfunc_selector(CGameScene::closeAlert));
	    	this ->runAction(CCSequence::create(deleyAct,callsa , NULL));
}
void CGameScene::closeAlert(CCObject* pSender){

	CCScene * rscene = CCDirector::sharedDirector()->getRunningScene();
	this->removeChildByTag(212121);
	this->removeChildByTag(212121+1);
}
