//
//  Player.cpp
//  MyRun
//
//  Created by Mac on 14-7-25.
//
//

#include "Player.h"

#include "SoundManager.h"
#include "GameData.h"
#include "GameModel.h"

#define PLAYER_INITIAL_SPEED 4
#define PLAYER_JUMP 17
#define GRAVITY 1.2f
#define AIR_FRICTION 0.6f
#define TERMINAL_VELOCITY 70

CPlayer::CPlayer()
{
    m_fAcceleration = 0;
    m_fMaxSpeed     = 0;
    m_fMoveSpeed    = 5;
    m_iGravity      = 20;
//    m_iVIT          = 10;
    
    m_bDying = false;
    m_bResting = false;
    m_bTakingDamage = false;
    m_bSlowDown = false;
    m_bSliping = false;
    m_bIsAccelerate = false;
    m_bIsAbsorb = false;
    m_bIsGiant = false;
    m_iShield = 0;
    
    nSlowDown_Duration   = 0;
    n_AccelerateDuration = 0;
    n_AbsorbDuration     = 0;
    n_GiantDuration      = 0;
    
    m_iAccelerateLevel   = 0;
    m_iAbsorbLevel       = 0;
    m_iGiantLevle        = 0;
    
    _screenSize = CCDirector::sharedDirector()->getWinSize();
    m_vNextPosition = CCPointZero;
    
    m_pSlowDownEmitter = NULL;
    m_pShieldEmitter = NULL;
    m_pSpeedEmitter = NULL;
    m_pGoldEmitter = NULL;
    
    m_fVITCounter = 0;
    
    m_iPlayerAction = kActionFalling;
}

CPlayer::~CPlayer()
{
    
}

CPlayer* CPlayer::createPlayer(string szName)
{
    CPlayer* pPlayer = new CPlayer;
    if (pPlayer && pPlayer->init())
    {
        pPlayer->initPlayer(szName);
        pPlayer->autorelease();
        return pPlayer;
    }
    CC_SAFE_DELETE(pPlayer);
    return NULL;
}

void CPlayer::initPlayer(string szName)
{
    m_szSpriteName = szName;
    
    m_pTailSprite = CCSprite::createWithSpriteFrameName("Character1_TailFall_1.png");
    m_pTailSprite->setPosition(ccp(112, 100));
    addChild(m_pTailSprite,-1);
    
    m_pEyesSprite = CCSprite::createWithSpriteFrameName("Character1_EyesRun_1.png");
    m_pEyesSprite->setPosition(ccp(112, 100));
    m_pEyesSprite->setVisible(false);
    addChild(m_pEyesSprite,2);
    
    m_pEffectSprite = CCSprite::create("blankpic.png");
    m_pEffectSprite->setPosition(ccp(72, 135));
    addChild(m_pEffectSprite);
    this->setAnchorPoint(ccp(0.5f, 0.0f));
    
//    m_pSpeedEmitter = CCParticleSystemQuad::create("speedLine11.plist");
//    this->addChild(m_pSpeedEmitter, 106);
//    m_pSpeedEmitter->setPosition(ccp(122, 100));
//    m_pSpeedEmitter->setAutoRemoveOnFinish(true);
     
    this->setZOrder(1);
}

void CPlayer::setData(string szName,int iID,int iHP,float iAcceleration, float iMaxSpeed)
{
    m_szCharacterName = szName;
	m_iID = iID;
	m_iMaxHP = iHP;
	m_iHP = iHP;
    m_iMaxVIT = 100;
    m_iVIT = m_iMaxVIT;
    
    m_fAcceleration = iAcceleration;
    m_fMaxSpeed = iMaxSpeed;
    
    m_iPlayerState = kPlayerMoving;
    m_bJumping = false;
    m_bDoubleJumping = false;
    
    m_fWidth = 110;
    m_fHeight = 110;
        
    m_fCollectRadiusX = 0;
    m_fCollectRadiusY = 0;
}

void CPlayer::initAnimation(vector<string> szAction,vector<float> delay, int max_frame)
{
    CCArray *animFrames = CCArray::create();
	string szframeName;
	string szActionName;
    
	for(int i=0;i<szAction.size();i++)
	{
        animFrames->removeAllObjects();
        
        //创建动画每一帧，从cache里面读取
        char str[3] = {0};
        for(int j = 1; j <max_frame; j++)
        {
            sprintf(str, "%d", j);
            szframeName = m_szSpriteName + szAction[i] + str + ".png";
            CCSpriteFrame* frame = CCSpriteFrameCache::sharedSpriteFrameCache()->spriteFrameByName( szframeName.c_str() );
            if(!frame) break;
            animFrames->addObject(frame);
        }
        if (szAction[i] == "EyesRun_") {
            szframeName = m_szSpriteName + szAction[i] + "1" + ".png";
            CCSpriteFrame* frame = CCSpriteFrameCache::sharedSpriteFrameCache()->spriteFrameByName( szframeName.c_str() );
            if(!frame) break;
            animFrames->addObject(frame);
        }
        if (szAction[i] == "EyesSlip_") {
            szframeName = m_szSpriteName + szAction[i] + "1" + ".png";
            CCSpriteFrame* frame = CCSpriteFrameCache::sharedSpriteFrameCache()->spriteFrameByName( szframeName.c_str() );
            if(!frame) break;
            animFrames->addObject(frame);
        }
        szActionName = m_szSpriteName + szAction[i];
        CCAnimation* animation = CCAnimation::createWithSpriteFrames(animFrames,delay[i]);
        CCAnimationCache::sharedAnimationCache()->addAnimation(animation, szActionName.c_str());
	}
}
void CPlayer::initEfect(vector<string> szAction,vector<float> delay, int max_frame)
{
    CCArray *animFrames = CCArray::create();
	string szframeName;
	string szActionName;
    
	for(int i=0;i<szAction.size();i++)
	{
        animFrames->removeAllObjects();
        
        //创建动画每一帧，从cache里面读取
        char str[3] = {0};
        for(int j = 1; j <max_frame; j++)
        {
            sprintf(str, "%d", j);
            szframeName = szAction[i] + str + ".png";
            CCSpriteFrame* frame = CCSpriteFrameCache::sharedSpriteFrameCache()->spriteFrameByName( szframeName.c_str() );
            if(!frame) break;
            animFrames->addObject(frame);
        }
        szActionName = szAction[i];
        CCAnimation* animation = CCAnimation::createWithSpriteFrames(animFrames,delay[i]);
        CCAnimationCache::sharedAnimationCache()->addAnimation(animation, szActionName.c_str());
	}
}

void CPlayer::playAction(string szActing, bool bRepeat)
{
    this->stopActionByTag(1);
    
    string szBodyAction;
    if (m_bIsGiant) {
        szBodyAction = m_szSpriteName + "Iron" + szActing;
    }
    else{
        szBodyAction = m_szSpriteName + szActing;
    }
    
	if(bRepeat)
	{
        CCAction* act = CCRepeatForever::create(CCAnimate::create(CCAnimationCache::sharedAnimationCache()
                                                                  ->animationByName(szBodyAction.c_str())));
        act->setTag(1);
		this->runAction(act);
	}
	else
	{
        CCAction* act = CCAnimate::create(CCAnimationCache::sharedAnimationCache()
                                          ->animationByName(szBodyAction.c_str()));
        act->setTag(1);
		this->runAction(act);
	}
    
    
    if (szActing == "Fall_") {
        m_pEyesSprite->setVisible(false);
    }
    
    if (szActing == "Roll_" || szActing == "Shock_" || szActing == "Die_" || szActing == "Burn_") {
        m_pTailSprite->cocos2d::CCNode::setVisible(false);
        m_pTailSprite->stopAllActions();
    }
    else
    {
        string szTailAction = m_szSpriteName + "Tail" + szActing;
        CCAnimate* animTail = NULL;
        animTail = CCAnimate::create(CCAnimationCache::sharedAnimationCache()->animationByName(szTailAction.c_str()));
        CCAction* actTail = CCRepeatForever::create(animTail);
        m_pTailSprite->setVisible(true);
        m_pTailSprite->stopAllActions();
        m_pTailSprite->runAction(actTail);
    }
    
    if (szActing == "Run_")
    {
        if (m_bIsGiant)
        {
            m_pEyesSprite->setVisible(false);
        }
        else
        {
            m_pEyesSprite->setVisible(true);
            string szTailAction = m_szSpriteName + "Eyes" + szActing;
            CCAction* actEyes = NULL;
            actEyes = CCRepeatForever::create( CCSequence::create(
                                                                  CCAnimate::create(
                                                                                    CCAnimationCache::sharedAnimationCache()
                                                                                    ->animationByName(szTailAction.c_str())),
                                                                  CCDelayTime::create(2),
                                                                  NULL) );
            m_pEyesSprite->stopAllActions();
            m_pEyesSprite->runAction(actEyes);
            
            m_pEyesSprite->setPosition(ccp(112, 101));
            CCAction* actMoveEyes = CCRepeatForever::create( CCSequence::create(
                                                                                CCMoveBy::create(0.03, ccp(0, -2)),
                                                                                CCMoveBy::create(0.03, ccp(0, -2)),
                                                                                CCMoveBy::create(0.03, ccp(0, +2)),
                                                                                CCMoveBy::create(0.03, ccp(0, +2)),
                                                                                CCMoveBy::create(0.03, ccp(0, 0)),
                                                                                CCMoveBy::create(0.03, ccp(0, 0)),
                                                                                CCMoveBy::create(0.03, ccp(0, -2)),
                                                                                CCMoveBy::create(0.03, ccp(0, -2)),
                                                                                CCMoveBy::create(0.03, ccp(0, +2)),
                                                                                CCMoveBy::create(0.03, ccp(0, +2)),
                                                                                CCMoveBy::create(0.03, ccp(0, 0)),
                                                                                CCMoveBy::create(0.03, ccp(0, 0)),
                                                                                NULL));
            m_pEyesSprite->runAction(actMoveEyes);
        }
    }
    
    if (szActing == "Slip_")
    {
        m_pEyesSprite->setVisible(true);
        string szTailAction = m_szSpriteName + "Eyes" + szActing;
        CCAction* actEyes = NULL;
        actEyes =CCRepeatForever::create(  CCSequence::create(CCAnimate::create(
                                                                                CCAnimationCache::sharedAnimationCache()
                                                                                ->animationByName(szTailAction.c_str())),
                                                              CCDelayTime::create(2),
                                                              NULL)) ;
        m_pEyesSprite->stopAllActions();
        m_pEyesSprite->runAction(actEyes);
    }
    
    if (szActing == "Shock_") {
        m_pEyesSprite->stopAllActions();
        m_pEyesSprite->setVisible(false);
    }
    else if (szActing == "Die_"){
        m_pEyesSprite->stopAllActions();
        m_pEyesSprite->setVisible(false);
    }
    
}

void CPlayer::playEffect(string szActing, bool bRepeat)
{
    m_pEffectSprite->stopActionByTag(1);
	string szAction = szActing;
	if(bRepeat)
	{
        CCAction* act = CCRepeatForever::create(CCAnimate::create(CCAnimationCache::sharedAnimationCache()
                                                                  ->animationByName(szAction.c_str())));
        act->setTag(1);
		m_pEffectSprite->runAction(act);
	}
	else
	{
        CCAction* act = CCAnimate::create(CCAnimationCache::sharedAnimationCache()
                                          ->animationByName(szAction.c_str()));
        act->setTag(1);
		m_pEffectSprite->runAction(act);
	}
}




void CPlayer::update (float dt)
{
    
        if (m_fMoveSpeed + m_fAcceleration <= m_fMaxSpeed)
        {
            m_fMoveSpeed += m_fAcceleration;
        } else
        {
            m_fMoveSpeed = m_fMaxSpeed;
        }
        
        m_vector.x = m_fMoveSpeed;
        m_vNextPosition.x = this->getPositionX() + m_vector.x;
        this->setPositionX(m_vNextPosition.x);
        
        m_fVITCounter += 1;
        if (m_fVITCounter >= 40) {
            m_fVITCounter = 0;
            m_iVIT--;
            CCInteger* pVIT = CCInteger::create(m_iVIT);
            CCNotificationCenter::sharedNotificationCenter()->postNotification("updateVITBar",pVIT);
        }
        
        switch (m_iPlayerState)
        {
            case kPlayerMoving:
                m_vector.y -= GRAVITY;
                break;
            case kPlayerFalling:
                m_vector.y -= GRAVITY * AIR_FRICTION;
                break;
            case kPlayerFly:
                m_vector.y -= GRAVITY * (AIR_FRICTION+0.2);
                break;
//            case kPlayerDying:
//                m_vector.x = 0;
//                this->setPositionX(this->getPositionX() + m_vector.x);
//                break;
        }
        
        if (m_bJumping)
        {
            m_iPlayerState = kPlayerFalling;
            m_vector.y = PLAYER_JUMP;
            m_bJumping = false;
        }
        else if (m_bDoubleJumping)
        {
            m_iPlayerState = kPlayerFly;
            m_vector.y = PLAYER_JUMP * 1.1f;
            m_bDoubleJumping = false;
        }
        
        
        //加速
        if (m_bIsAccelerate && n_AccelerateDuration > 0 )
        {
            n_AccelerateDuration -= 10;
            
            if (n_AccelerateDuration <= 0)
            {
                recoverFromAccelerate();
            }
        }
        
        //吸金币
        if (m_bIsAbsorb && n_AbsorbDuration > 0)
        {
            n_AbsorbDuration -= 10;
            
            if (n_AbsorbDuration <= 0)
            {
                recoverFromAbsorb();
            }
        }
        
        //变大
        if (m_bIsGiant && n_GiantDuration > 0)
        {
            n_GiantDuration -= 10;
            
            if (n_GiantDuration <= 0)
            {
                recoverFromGiant();
            }
        }
        
        if (m_vector.y < -TERMINAL_VELOCITY)
            m_vector.y = -TERMINAL_VELOCITY;
        
        m_vNextPosition.y = this->getPositionY() + m_vector.y;
        if (m_vector.x * m_vector.x < 0.01)
            m_vector.x = 0;
        if (m_vector.y * m_vector.y < 0.01)
            m_vector.y = 0;
    
    if (m_vNextPosition.y <= -120) {
        m_bDying = true;
        m_vNextPosition.y = 950;
        CCNotificationCenter::sharedNotificationCenter()->postNotification("playerDie");
    }
}


CCRect CPlayer::getCollisionRect()
{
    if (m_bSliping)
    {
        return CCRect(this->getPositionX() - getContentSize().width * 0.1f, this->getPositionY(), 25, 55);
        
    }
    return CCRect(this->getPositionX() - getContentSize().width * 0.1f, this->getPositionY(), 55, 110);
}

CCRect CPlayer::getCollectRect()
{
    return CCRect(this->getPositionX(),
                  this->getPositionY() - m_fCollectRadiusY,
                  55  + m_fCollectRadiusX,
                  110 + m_fCollectRadiusY*2);
}

void CPlayer::takeDamage(int iDamage, int iElementType)
{
    m_bTakingDamage = true;
    scheduleOnce(schedule_selector(CPlayer::recoverFromDamage), 1.5);
    
    if (m_iShield > 0)
    {
        m_iShield -= iDamage;
        
        if (m_iShield == 0) {
            m_pShieldEmitter->stopSystem();
            
            CCParticleSystemQuad* m_pEmitter = CCParticleSystemQuad::create("shield_popo.plist");
            this->addChild(m_pEmitter,2);
            m_pEmitter->setPosition(ccp(112, 130));
        }
    }
    else
    {
        m_iHP -= iDamage;
        CCNotificationCenter::sharedNotificationCenter()->postNotification("updateHealthBar", CCInteger::create(m_iHP));
        
        if (m_iHP > 0)
        {
            //特效
            CCParticleSystemQuad * emitter = CCParticleSystemQuad::create("hurt.plist");
            this->addChild(emitter, 106);
            emitter->setPosition(ccp(182, 150));
            emitter->setAutoRemoveOnFinish(true);
            
            switch (iElementType) {
                case 1:
                    setColor(ccc3(220, 20, 60));
                    m_pTailSprite->setColor(ccc3(220, 20, 60));
                    break;
                case 2:
                    setColor(ccc3(220, 20, 60));
                    m_pTailSprite->setColor(ccc3(220, 20, 60));
                    break;
                case 3:
                    setColor(ccc3(220, 20, 60));
                    m_pTailSprite->setColor(ccc3(220, 20, 60));
                    break;
                case 4:
                    setColor(ccc3(220, 20, 60));
                    m_pTailSprite->setColor(ccc3(220, 20, 60));
                    break;
                case 5:
                    setColor(ccc3(128, 0, 128));
                    m_pTailSprite->setColor(ccc3(128, 0, 128));
                    break;
                default:
                    break;
            }
        }
        else
        {
            playAction("Fall_",false);
            m_bDying = true;
            if (m_vNextPosition.y >= m_fGroundY) {
                runAction( CCSequence::create(CCDelayTime::create(0.75),
                                              CCMoveTo::create(0.05, ccp(getPositionX(), m_fGroundY)),
                                              CCCallFuncN::create(this, callfuncN_selector(CPlayer::doDieAction)),
                                              NULL));
                
//            runAction( CCSequence::create(CCMoveBy::create(0.25, ccp(0, 150)),
//                                          CCMoveBy::create(0.75, ccp(0, -720)),
//                                          NULL) );
            }
            switch (iElementType) {
                case 1:
                    playAction("Burn_");
                    break;
                case 2:
                    playAction("Shock_");
                    break;
                case 4:
                    playAction("Burn_");
                    break;
                case 5:
                    playAction("Shock_");
                    break;
                default:
                    playAction("Shock_");
                    break;
            }
            
            m_fMoveSpeed = -1;
            //            CCAction* pAct = CCSequence::create(CCDelayTime::create(0.35),
            //                                                CCCallFuncN::create(this, callfuncN_selector(CPlayer::doDie)),
            //                                                NULL);
            //            runAction(pAct);
            
            CCNotificationCenter::sharedNotificationCenter()->postNotification("playerDie");
        }
    }
    
}

void CPlayer::recoverFromDamage(CCObject* pSender)
{
    m_bTakingDamage = false;
    
    setColor(ccc3(255, 255, 255));
    m_pTailSprite->setColor(ccc3(255, 255, 255));
}

void CPlayer::slowDown(float Time)
{
    if (!m_bSlowDown)
    {
        m_fMoveSpeed = m_fMoveSpeed*0.5f;
        m_fMaxSpeed = m_fMaxSpeed * 0.5f;
        m_bSlowDown = true;
    }
    scheduleOnce(schedule_selector(CPlayer::recoverFromSlowDown), Time);
    
    //特效
    if (m_pSlowDownEmitter == NULL)
    {
        m_pSlowDownEmitter = CCParticleSystemQuad::create("speedDown.plist");
        this->addChild(m_pSlowDownEmitter,2);
        m_pSlowDownEmitter->setPosition(ccp(162, 110));
    }
    else
    {
        m_pSlowDownEmitter->resetSystem();
    }
    
}

void CPlayer::recoverFromSlowDown()
{
    m_fMoveSpeed = m_fMoveSpeed * 2;
    m_fMaxSpeed = m_fMaxSpeed * 2;
    m_bSlowDown = false;
    
    m_pSlowDownEmitter->stopSystem();
    removeChild(m_pSlowDownEmitter, true);
    m_pSlowDownEmitter = NULL;
}

void CPlayer::doJump()
{
    string szJump;
    string szFall;
    
    if (m_bIsGiant) {
        szJump = getSpriteName() + "IronJump_";
    }
    else{
        szJump = getSpriteName() + "Jump_";
    }
    
    if (m_bIsGiant) {
        szFall = getSpriteName() + "IronFall_";
    }
    else{
        szFall = getSpriteName() + "Fall_";
    }
    CCAnimate* pJump = CCAnimate::create(CCAnimationCache::sharedAnimationCache()
                                         ->animationByName(szJump.c_str()));
    CCAnimate* pFall = CCAnimate::create(CCAnimationCache::sharedAnimationCache()
                                         ->animationByName(szFall.c_str()));
    CCAction* pAct = CCSequence::create(pJump,pFall,NULL);
    pAct->setTag(1);
    runAction(pAct);
    
    string szTailJump = getSpriteName() + "TailJump_";
    string szTailFall = getSpriteName() + "TailFall_";
    CCAnimate* pTailJump = CCAnimate::create(CCAnimationCache::sharedAnimationCache()
                                             ->animationByName(szTailJump.c_str()));
    
    CCAction* pTailAct = CCSequence::create(pTailJump,CCCallFuncN::create(this, callfuncN_selector(CPlayer::showTail)),NULL);
    m_pTailSprite->stopAllActions();
    m_pTailSprite->runAction(pTailAct);
    
    m_pEyesSprite->setVisible(false);

    m_iPlayerAction = kActionJumping;
}

void CPlayer::doDoubleJump()
{
    string szJump;
    string szFall;
    
    if (m_bIsGiant) {
        szJump = getSpriteName() + "IronRoll_";
    }
    else{
        szJump = getSpriteName() + "Roll_";
    }
    
    if (m_bIsGiant) {
        szFall = getSpriteName() + "IronFall_";
    }
    else{
        szFall = getSpriteName() + "Fall_";
    }
    
    CCAnimate* pJump = CCAnimate::create(CCAnimationCache::sharedAnimationCache()
                                         ->animationByName(szJump.c_str()));
    CCAnimate* pFall = CCAnimate::create(CCAnimationCache::sharedAnimationCache()
                                         ->animationByName(szFall.c_str()));
    CCAction* pAct = CCSequence::create(pJump,pFall,NULL);
    pAct->setTag(1);
    runAction(pAct);
    
    string szTailJump = getSpriteName() + "Roll_";
    
    CCAnimate* pTailJump = CCAnimate::create(CCAnimationCache::sharedAnimationCache()
                                             ->animationByName(szTailJump.c_str()));
    
    CCAction* pTailAct = CCSequence::create(CCCallFuncN::create(this, callfuncN_selector(CPlayer::hideTail)),
                                            pTailJump,
                                            CCCallFuncN::create(this, callfuncN_selector(CPlayer::showTail)),
                                            NULL);
    m_pTailSprite->stopAllActions();
    m_pTailSprite->runAction(pTailAct);
    
    m_pEyesSprite->setVisible(false);
    
    m_iPlayerAction = kActionDoubleJumping;
}

void CPlayer::doSlip()
{
    
    playAction("Slip_",false);
    
    m_bSliping = true;
    
    m_iPlayerAction = kActionSliping;
}

void CPlayer::recoverFromSlip()
{
    if (m_vector.y <=0 ) {
        
        playAction("Run_");
        
    }
    m_bSliping = false;
    m_iPlayerAction = kActionMoving;
}

void CPlayer::doDie()
{
    m_bDying = true;
}
void CPlayer::getItem(int iType)
{
    switch (iType)
    {
        case 1:
        {
            CSoundManager::getInstance()->playSound("sound/rush.ogg");
            
            toAccelerate();
            toAbsorb();
            
//            if (CGameData::getInstance()->getFood()) {
//                n_AccelerateDuration = (1800 + m_iAccelerateLevel*60)*2;
//                n_TotalDuration = 4800;
//            }
//            else{
//                n_AccelerateDuration = 1800 + m_iAccelerateLevel*60;
//                n_TotalDuration = 2400;
//            }
            
            n_AccelerateDuration = (1800 + m_iAccelerateLevel*60 )* GameModel::getInstance()->usingPet;
            n_TotalDuration =  2400 * GameModel::getInstance()->usingPet ;
            
            
            CCDictionary* pDic = CCDictionary::create();
            pDic->setObject(CCInteger::create(n_AccelerateDuration), "Duration");
            pDic->setObject(CCInteger::create(n_TotalDuration), "TotalDuration");
            CCNotificationCenter::sharedNotificationCenter()->postNotification("accelerateCountDown",pDic);
        }
            break;
        case 2:
        {
            toAbsorb();
            
            n_AbsorbDuration = 2400 + m_iAbsorbLevel*60;
            
            CCDictionary* pDic = CCDictionary::create();
            pDic->setObject(CCInteger::create(n_AbsorbDuration), "Duration");
            pDic->setObject(CCInteger::create(3000), "TotalDuration");
            CCNotificationCenter::sharedNotificationCenter()->postNotification("absortCountDown",pDic);
        }
            break;
        case 3:
        {
            toGiant();
            
            n_GiantDuration = 4800 + m_iGiantLevle*120;
            
            CCDictionary* pDic = CCDictionary::create();
            pDic->setObject(CCInteger::create(n_GiantDuration), "Duration");
            pDic->setObject(CCInteger::create(6000), "TotalDuration");
            CCNotificationCenter::sharedNotificationCenter()->postNotification("giantCountDown",pDic);
        }
            break;
        case 4:
            CSoundManager::getInstance()->playSound("sound/UCfast.mp3");
            
            regenerate();
            break;
        case 5:
            CSoundManager::getInstance()->playSound("sound/door_cross.ogg");
            
            CCNotificationCenter::sharedNotificationCenter()->postNotification("Portal");        

        default:
            break;
    }
}

void useItem(int iType, int extra)
{
    
}

void CPlayer::getGold()
{
    if (CGameData::getInstance()->getEffectEnable())
    {
        //特效
        if (m_pGoldEmitter == NULL) {
            m_pGoldEmitter = CCParticleSystemQuad::create("getGold6.plist");
            this->addChild(m_pGoldEmitter);
            m_pGoldEmitter->setPosition(ccp(182, 150));
        }
        else
        {
            m_pGoldEmitter->resetSystem();
        }
        
    }
    CSoundManager::getInstance()->playSound("sound/get_gold.ogg");        
}



void CPlayer::toAccelerate()
{
    if (n_AccelerateDuration > 0)
    {
        n_AccelerateDuration = 0;
        
        m_fAcceleration = m_fMaxSpeed /2;
        m_fMaxSpeed = m_fMaxSpeed /2;
        m_fCollectRadiusX = 0;
        m_fCollectRadiusY = 0;
        m_bIsAccelerate = false;
        
        removeChild((CCSprite*)getChildByTag(123), true);
    }
    
    m_bIsAccelerate = true;
    m_fAcceleration = m_fMaxSpeed * 2;
    m_fMaxSpeed = m_fMaxSpeed * 2;
    m_fCollectRadiusX = 600;
    m_fCollectRadiusY = 400;
    
    playAction("Fall_",false);
    
    CCSprite* pEffect = CCSprite::create("blankpic.png");
    pEffect->setPosition(ccp(-70, 125));
    addChild(pEffect, -2, 123);
    CCAction* pAct2 = NULL;
    pAct2 = CCRepeatForever::create(CCAnimate::create(CCAnimationCache::sharedAnimationCache()->animationByName("EffectSpeedUp")));
    pEffect->runAction(pAct2);
    
    getShield(1);
}

void CPlayer::recoverFromAccelerate()
{
    n_AccelerateDuration = 0;
    
    m_fAcceleration = m_fMaxSpeed /2;
    m_fMaxSpeed = m_fMaxSpeed /2;
    m_fCollectRadiusX = 0;
    m_fCollectRadiusY = 0;
    m_bIsAccelerate = false;
    
    removeChild((CCSprite*)getChildByTag(123), true);
    
    playAction("Fall_",false);
    
    CCNotificationCenter::sharedNotificationCenter()->postNotification("showBridge");
}

void CPlayer::toAbsorb()
{
    
    m_fCollectRadiusX = 600;
    m_fCollectRadiusY = 200;
    m_bIsAbsorb = true;
    
}

void CPlayer::recoverFromAbsorb()
{
    n_AbsorbDuration = 0;
    
    m_fCollectRadiusX = 0;
    m_fCollectRadiusY = 0;
    m_bIsAbsorb = false;
}

void CPlayer::toGiant()
{
    m_fAcceleration = m_fMaxSpeed * 1.3;
    m_fMaxSpeed = m_fMaxSpeed * 1.3;
    m_fCollectRadiusX = 110;
    m_fCollectRadiusY = 55;
    m_bIsGiant = true;
    runAction(CCScaleTo::create(0.75f, 1.6));
    
    stopActionByTag(1);
    
    switch (getPlayerAction())
    {
        case kPlayerMoving:
            playAction("Run_");
            break;
        case kActionJumping:
            playAction("Jump_",false);
            break;
        case kActionDoubleJumping:
            playAction("Roll_");
            break;
        case kActionFalling:
            playAction("Fall_",false);
            break;
        case kActionSliping:
            playAction("Slip_");
            break;
        default:
            break;
    }
    
}

void CPlayer::recoverFromGiant()
{
    n_GiantDuration = 0;
    m_fAcceleration = m_fMaxSpeed /1.3;
    m_fMaxSpeed = m_fMaxSpeed /1.3;
    m_fCollectRadiusX = 0;
    m_fCollectRadiusY = 0;
    m_bIsGiant = false;
    runAction(CCScaleTo::create(0.75f, 1));
}

void CPlayer::regenerate()
{
    m_bResting = true;
    
    if (m_iHP < m_iMaxHP) {
        m_iHP++;
    }
    m_iVIT = m_iMaxVIT;
    CCNotificationCenter::sharedNotificationCenter()->postNotification("updateHealthBar", CCInteger::create(m_iHP));
    CCNotificationCenter::sharedNotificationCenter()->postNotification("updateVITBar", CCInteger::create(m_iVIT));
    
    
    CCSprite* pEffect = CCSprite::create("blankpic.png");
//    CCAction* pAct2  = CCSequence::create(CCAnimate::create(CCAnimationCache::sharedAnimationCache()->animationByName("EffectRebirth")),
//                                          CCCallFuncN::create(this, callfuncN_selector(CPlayer::removeEffect)),
//                                          NULL);
    CCAction* pAct2  =  CCRepeatForever::create(CCAnimate::create(CCAnimationCache::sharedAnimationCache()->animationByName("EffectRebirth")));
    pEffect->setPosition(ccp(100, 150));
    pEffect->setAnchorPoint(ccp(0.5f, 0.4f));
    pEffect->setTag(10086);
    addChild(pEffect,2);
    pEffect->runAction(pAct2);
    
    scheduleOnce(schedule_selector(CPlayer::completeRegenerate), 3);
    
    CCNotificationCenter::sharedNotificationCenter()->postNotification("TakeABreak");
}

void CPlayer::completeRegenerate()
{
    m_bResting = false;
    
    this->removeChildByTag(10086);
}

void CPlayer::getShield(int iShieldCount)
{
    if (m_iShield == 0)
    {
        m_pShieldEmitter = CCParticleSystemQuad::create("shield.plist");
        this->addChild(m_pShieldEmitter,2);
        m_pShieldEmitter->setPosition(ccp(112, 130));
    }
    
    m_iShield = iShieldCount;
    
    
}

void CPlayer::removeEffect(CCObject* pSender)
{
    this->removeChild((CCSprite*)pSender,true);
}

void CPlayer::showTail(CCObject* pSender)
{
    m_pTailSprite->setVisible(true);
    
    string szTailFall = getSpriteName() + "TailFall_";
    CCAction* pTailFall = CCRepeatForever::create(CCAnimate::create(CCAnimationCache::sharedAnimationCache()
                                             ->animationByName(szTailFall.c_str())));
    m_pTailSprite->stopAllActions();
    m_pTailSprite->runAction(pTailFall);
    
    m_iPlayerAction = kActionFalling;
}

void CPlayer::hideTail(CCObject* pSender)
{
    m_pTailSprite->setVisible(false);
}

void CPlayer::doDieAction()
{
    playAction("Die_",false);
}
