//
//  Player.h
//  MyRun
//
//  Created by Mac on 14-7-25.
//
//

#ifndef __MyRun__Player__
#define __MyRun__Player__

#include "cocos2d.h"
//#include "BaseCharacter.h"
USING_NS_CC;
using namespace std;

enum PlayerState
{
	kPlayerMoving,//主角在移动
	kPlayerFalling,//主角在降落
	kPlayerDying,//主角死亡
    kPlayerFly,
};

enum PlayerAction
{
    kActionMoving,
    kActionSliping,
    kActionJumping,
    kActionDoubleJumping,
    kActionFalling,
};



class CPlayer : public CCSprite
{
public:
    CPlayer();
    ~CPlayer();
    
    
    static CPlayer* createPlayer(string szName);
    void initPlayer(string szName);
    void setData(string szName,int iID,int iHP,float iAcceleration, float iMaxSpeed);
    void initAnimation(vector<string> szAction,vector<float> delay, int max_frame);
    void initEfect(vector<string> szAction,vector<float> delay, int max_frame);
    
    void playAction(string szActing, bool bRepeat = true);
    void playEffect(string szActing, bool bRepeat = true);
    
    virtual void update (float dt);
    void takeDamage(int iDamage, int iElementType);
    void recoverFromDamage(CCObject* pSender);
    void slowDown(float Time);
    void recoverFromSlowDown();
    
    void doJump();
    void doDoubleJump();
    void doSlip();
    void recoverFromSlip();
    void doDie();
    
    void getItem(int iType);
    void getGold();
    void useItem(int iType, int extra);
    
    void toAccelerate();
    void recoverFromAccelerate();
    void toAbsorb();
    void recoverFromAbsorb();
    void toGiant();
    void recoverFromGiant();
    void regenerate();
    void completeRegenerate();
    void getShield(int iShieldCount);
    
    void removeEffect(CCObject* pSender);
    void showTail(CCObject* pSender);
    void hideTail(CCObject* pSender);
    
    void doDieAction();
    
    //主角位置的真正实现
	inline virtual void place ()
    {
		this->setPositionY( m_vNextPosition.y );
		if (m_vector.x > 0 && this->getPositionX()-m_fMoveDistance < _screenSize.width * 0.3f)
        {
			this->setPositionX(this->getPositionX() + m_vector.x);
		}
	};
    
	//主角在X轴上的位置[左侧有效碰撞]
	inline int left() {
		return this->getPositionX() - m_fWidth * 0.5f;
	}
    
	//主角在X轴上的位置[右侧有效碰撞]
	inline int right() {
		return this->getPositionX() + m_fWidth * 0.5f;
	}
    
	//主角在Y轴上的位置
	inline int top() {
		return this->getPositionY() + 55;
	}
    
	//主角在Y轴上的位置【底部有效碰撞】
	inline int bottom() {
		return this->getPositionY() ;
	}
    
	//主角下一个位置的左侧
	inline int next_left() {
		return m_vNextPosition.x - m_fWidth * 0.5f;
	}
    
	//主角下一个位置的右侧
	inline int next_right() {
		return m_vNextPosition.x + m_fWidth * 0.5f;
	}
    
	//主角下一个位置的上部
	inline int next_top() {
        if(m_bSliping){
            return m_vNextPosition.y + 55;
        }
        else
        {
            return m_vNextPosition.y + m_fHeight;
        }
	}
    
	//主角下一个位置的底部
	inline int next_bottom() {
		return m_vNextPosition.y;
	}
    
    CCRect getCollisionRect();
    CCRect getCollectRect();
    
private:
    float m_fAcceleration;
    
    int m_iGravity;
    
    CC_SYNTHESIZE(string,m_szSpriteName, SpriteName);
    CC_SYNTHESIZE(float, m_fGroundY, GroundY);
    //状态
    CC_SYNTHESIZE(bool,  m_bDying, Dying);//死亡
    CC_SYNTHESIZE(bool,  m_bResting, Resting);//休息
    CC_SYNTHESIZE(bool,  m_bJumping, Jumping);//跳
    CC_SYNTHESIZE(bool,  m_bDoubleJumping, DoubleJumping);//两连跳
    CC_SYNTHESIZE(bool,  m_bSliping, Sliping);//铲地
    CC_SYNTHESIZE(bool,  m_bTakingDamage, TakingDamage);//受伤
    CC_SYNTHESIZE(bool,  m_bSlowDown, SlowDown);//减速
    CC_SYNTHESIZE(bool,  m_bIsAccelerate, IsAccelerate);//加速
    CC_SYNTHESIZE(bool,  m_bIsAbsorb, IsAbsorb);//吸金币
    CC_SYNTHESIZE(bool,  m_bIsGiant, IsGiant);//变大
    CC_SYNTHESIZE(int,   m_iShield, Shield);
    //属性
    CC_SYNTHESIZE(int,   m_iMaxHP, MaxHP);
	CC_SYNTHESIZE(int,   m_iHP, HP);
    CC_SYNTHESIZE(int,   m_iMaxVIT, MaxVIT);
	CC_SYNTHESIZE(int,   m_iVIT, VIT);
    CC_SYNTHESIZE(int,   m_iPlayerState,PlayerState);
    CC_SYNTHESIZE(float, m_fMoveSpeed, MoveSpeed);
    CC_SYNTHESIZE(float, m_fMoveDistance, MoveDistance);
    CC_SYNTHESIZE(float, m_fMaxSpeed, MaxSpeed);
    
    CC_SYNTHESIZE(float, m_fWidth, Width);
	CC_SYNTHESIZE(float, m_fHeight, Height);
    CC_SYNTHESIZE(float, m_fCollectRadiusX, CollectRadiusX);
    CC_SYNTHESIZE(float, m_fCollectRadiusY, CollectRadiusY);
    CC_SYNTHESIZE(CCPoint, m_vNextPosition, NextPosition);
    CC_SYNTHESIZE(CCPoint, m_vector, Vector);
    
    //升级
    CC_SYNTHESIZE(int, m_iAccelerateLevel, AccelerateLevel);
    CC_SYNTHESIZE(int, m_iAbsorbLevel, AbsorbLevel);
    CC_SYNTHESIZE(int, m_iGiantLevle, GiantLevle);
    
    CC_SYNTHESIZE(int, m_iPlayerAction, PlayerAction);
    
    CCSize _screenSize;
    
    string              m_szCharacterName;
    int					m_iID;
    
    map<string, CCAction*> _Action_Map;
    
    int nSlowDown_Duration;
    int n_AccelerateDuration;
    int n_AbsorbDuration;
    int n_GiantDuration;
    
    int n_TotalDuration;

    CCSprite*           m_pTailSprite;
    CCSprite*           m_pEffectSprite;
    CCSprite*           m_pEyesSprite;
    
    CCParticleSystemQuad* m_pSlowDownEmitter;
    CCParticleSystemQuad* m_pShieldEmitter;
    CCParticleSystemQuad* m_pSpeedEmitter;
    CCParticleSystemQuad* m_pGoldEmitter;
    
    float               m_fVITCounter;
    
    
    
    
public:
    
    
};

#endif /* defined(__MyRun__Player__) */
