//
//  SettingLayer.cpp
//  runTileMap
//
//  Created by Mac on 14-10-23.
//
//

#include "SettingLayer.h"
#include "SoundManager.h"
#include "GameData.h"

CSettingLayer::CSettingLayer()
{
    pBGSprite = NULL;
    pTextSprite = NULL;
    pSoundControl = NULL;
    pMusicControl = NULL;
    pEffectControl = NULL;
    pMenu = NULL;
}

CSettingLayer::~CSettingLayer()
{
    
}

bool CSettingLayer::init()
{
    if (CCLayer::init( ) )
	{
        pBGSprite = CCSprite::createWithSpriteFrameName("ui_main_Setting.png");
        pBGSprite->setAnchorPoint(CCPointZero);
        pBGSprite->setPosition(ccp(0, 0));
        addChild(pBGSprite);
        
        pTextSprite = CCSprite::createWithSpriteFrameName("ui_main_SettingText.png");
        pTextSprite->setAnchorPoint(CCPointZero);
        pTextSprite->setPosition(ccp(0, 0));
        addChild(pTextSprite);
        
        CCSpriteFrameCache* cache = CCSpriteFrameCache::sharedSpriteFrameCache();
        
        CCMenuItemToggle* pSoundItem = createToggle(menu_selector(CSettingLayer::setSound),"btn-main_switchOff.png","btn-main_switchOn.png");
        pSoundItem->setSelectedIndex(CGameData::getInstance()->getSoundEnable());
        pSoundItem->setPosition( ccp(260,340) );
        
        CCMenuItemToggle* pMusicItem = createToggle(menu_selector(CSettingLayer::setMusic),"btn-main_switchOff.png","btn-main_switchOn.png");
        pMusicItem->setSelectedIndex(CGameData::getInstance()->getMusicEnable());
        pMusicItem->setPosition( ccp(260,280) );
        
        CCMenuItemToggle* pEffectItem = createToggle(menu_selector(CSettingLayer::setEffect),"btn-main_switchOff.png","btn-main_switchOn.png");
        pEffectItem->setSelectedIndex(CGameData::getInstance()->getEffectEnable());
        pEffectItem->setPosition( ccp(260,220) );
        
        CCMenuItemImage *pCloseItem = CCMenuItemImage::create();
        pCloseItem->setNormalSpriteFrame(cache->spriteFrameByName("btn_main_close.png"));
        pCloseItem->setSelectedSpriteFrame(cache->spriteFrameByName("btn_main_close.png"));
        pCloseItem->initWithTarget(this, menu_selector(CSettingLayer::closeLayer) );
        pCloseItem->setPosition( ccp(330,415) );
        
        pMenu = CCMenu::create(pSoundItem, pMusicItem, pEffectItem, pCloseItem, NULL);
        pMenu->setAnchorPoint(CCPointZero);
        pMenu->setPosition( ccp(0, 0) );
        addChild(pMenu, 2);
        
        return true;
    }
    
    return false;
}

void CSettingLayer::setMusic(CCObject* pSender)
{
    CSoundManager::getInstance()->playSound("sound/ui_button.ogg");
    
    CCMenuItemToggle* pToggle = (CCMenuItemToggle*)pSender;
    
    if (pToggle->getSelectedIndex()==1)
    {
        pToggle->setSelectedIndex(1);
        CSoundManager::getInstance()->setMusicOn(true);
        CGameData::getInstance()->setMusicEnable(true);
    }
    else
    {
        pToggle->setSelectedIndex(0);
        CSoundManager::getInstance()->setMusicOn(false);
        CGameData::getInstance()->setMusicEnable(false);
    }
}

void CSettingLayer::setSound(CCObject* pSender)
{
    CSoundManager::getInstance()->playSound("sound/ui_button.ogg");
    
    CCMenuItemToggle* pToggle = (CCMenuItemToggle*)pSender;
    
    if (pToggle->getSelectedIndex()==1)
    {
        pToggle->setSelectedIndex(1);
        CSoundManager::getInstance()->setSoundOn(true);
        CGameData::getInstance()->setSoundEnable(true);
    }
    else
    {
        pToggle->setSelectedIndex(0);
        CSoundManager::getInstance()->setSoundOn(false);
        CGameData::getInstance()->setSoundEnable(false);
    }
}

void CSettingLayer::setEffect(CCObject* pSender)
{
    CSoundManager::getInstance()->playSound("sound/ui_button.ogg");
    
    CCMenuItemToggle* pToggle = (CCMenuItemToggle*)pSender;
    
    if (pToggle->getSelectedIndex()==1)
    {
        pToggle->setSelectedIndex(1);
        CGameData::getInstance()->setEffectEnable(true);
    }
    else
    {
        pToggle->setSelectedIndex(0);
        CGameData::getInstance()->setEffectEnable(false);
    }
}

void CSettingLayer::closeLayer(CCObject* pSender)
{
    CGameData::getInstance()->updateData();
    
    CSoundManager::getInstance()->playSound("sound/ui_button.ogg");
    
    CCNotificationCenter::sharedNotificationCenter()->postNotification("showItemTable");
    this->removeFromParentAndCleanup(true);
}

void CSettingLayer::removeLayer()
{
    this->removeFromParentAndCleanup(true);
}

CCMenuItemToggle* CSettingLayer::createToggle(SEL_MenuHandler selector, char * fileNameNormal, char * fileNameSelect)
{
    CCSpriteFrameCache* cache = CCSpriteFrameCache::sharedSpriteFrameCache();
    
    CCMenuItemImage* imgNormal = CCMenuItemImage::create();
    imgNormal->setNormalSpriteFrame(cache->spriteFrameByName(fileNameNormal));
    CCMenuItemImage* imgSelect = CCMenuItemImage::create();
    imgSelect->setNormalSpriteFrame(cache->spriteFrameByName(fileNameSelect));
    CCMenuItemToggle* imgToggle = CCMenuItemToggle::createWithTarget(this, selector, imgNormal, imgSelect, NULL);
    return imgToggle;
}
